=== modified file 'CHANGES'
--- CHANGES	2002-04-23 23:16:43 +0000
+++ CHANGES	2004-06-11 13:51:34 +0000
@@ -1,3 +1,16 @@
+3.6.8 -> 3.7:
+	- always use compressext for the extension for compressed
+	  files; before compresscmd and compressext had to agree
+	- moved all compression to one code block
+	- compression, scripts don't use system() anymore
+	- compress and maillast didn't work together properly
+	- delaycompress and mailfirst didn't work properly
+	- don't use system() for mailing (or uncompressing) logs anymore
+	- use "-s" for speciying the subjected of mailed logs
+
+3.5.4 -> 3.6:
+        - See .spec file for changes
+
 3.5.3 -> 3.5.4:
 	- %defattr(-,root,root) in specfile
 

=== modified file 'Makefile'
--- Makefile	2002-04-23 23:16:43 +0000
+++ Makefile	2004-06-11 13:51:34 +0000
@@ -1,13 +1,67 @@
 VERSION = $(shell awk '/Version:/ { print $$2 }' logrotate.spec)
+CVSROOT = $(shell cat CVS/Root)
 CVSTAG = r$(subst .,-,$(VERSION))
-LFS = $(shell echo `getconf LFS_CFLAGS`)
-CFLAGS = -Wall -D_GNU_SOURCE -DVERSION=\"$(VERSION)\" $(RPM_OPT_FLAGS) $(LFS)
+OS_NAME = $(shell uname -s)
+LFS = $(shell echo `getconf LFS_CFLAGS 2>/dev/null`)
+CFLAGS = -Wall -D_GNU_SOURCE -D$(OS_NAME) -DVERSION=\"$(VERSION)\" $(RPM_OPT_FLAGS) $(LFS)
 PROG = logrotate
-BINDIR = /usr/sbin
-MANDIR = /usr/share/man
 MAN = logrotate.8
 LOADLIBES = -lpopt
 
+ifeq ($(WITH_SELINUX),yes)
+CFLAGS += -DWITH_SELINUX
+LOADLIBES += -lselinux 
+endif
+
+# HP-UX using GCC
+ifeq ($(OS_NAME),HP-UX)
+    ifeq ($(RPM_OPT_FLAGS),)
+        RPM_OPT_FLAGS = -O2
+    endif
+    CC = gcc
+    INSTALL = cpset
+    ifeq ($(POPT_DIR),)
+        POPT_DIR = /usr/local
+    endif
+    ifeq ($(HPLX_DIR),)
+	HPLX_DIR = /usr/local/hplx
+    endif
+    LOADLIBES += -lhplx -L$(HPLX_DIR)/lib
+    ifeq ($(BASEDIR),)
+	BASEDIR = /usr/local
+    endif
+endif
+
+# Solaris using gcc
+ifeq ($(OS_NAME),SunOS)
+    ifeq ($(RPM_OPT_FLAGS),)
+        RPM_OPT_FLAGS = -O2
+    endif
+    CC = gcc
+    INSTALL = install
+    ifeq ($(BASEDIR),)
+	BASEDIR = /usr/local
+    endif
+endif
+
+# Red Hat Linux
+ifeq ($(OS_NAME),Linux)
+    INSTALL = install
+    BASEDIR = /usr
+endif
+
+ifneq ($(POPT_DIR),)
+    CFLAGS += -I$(POPT_DIR)
+    LOADLIBES += -L$(POPT_DIR)
+endif
+
+ifneq ($(STATEFILE),)
+    CFLAGS += -DSTATEFILE=\"$(STATEFILE)\"
+endif
+
+BINDIR = $(BASEDIR)/sbin
+MANDIR = $(BASEDIR)/man
+
 #--------------------------------------------------------------------------
 
 OBJS = logrotate.o log.o config.o basenames.o
@@ -31,13 +85,13 @@
 $(PROG): $(OBJS)
 
 clean:
-	rm -f $(OBJS) $(PROG) core*
+	rm -f $(OBJS) $(PROG) core* .depend
 
 depend:
 	$(CPP) $(CFLAGS) -M $(SOURCES) > .depend
 
 .PHONY : test
-test:
+test: $(TARGET)
 	(cd test; ./test)
 
 install:
@@ -45,19 +99,24 @@
 	[ -d $(PREFIX)/$(MANDIR) ] || mkdir -p $(PREFIX)/$(MANDIR)
 	[ -d $(PREFIX)/$(MANDIR)/man8 ] || mkdir -p $(PREFIX)/$(MANDIR)/man8
 
-	install -s -m 755 $(PROG) $(PREFIX)/$(BINDIR)
-	install -m 644 $(MAN) $(PREFIX)/$(MANDIR)/man`echo $(MAN) | sed "s/.*\.//"`/$(MAN)
+	if [ "$(OS_NAME)" = HP-UX ]; then \
+	$(INSTALL) $(PROG) $(PREFIX)/$(BINDIR) 0755 bin bin; \
+	$(INSTALL) $(MAN) $(PREFIX)/$(MANDIR)/man`echo $(MAN) | sed "s/.*\.//"` 0644 bin bin; \
+	else \
+	$(INSTALL) -m 755 $(PROG) $(PREFIX)/$(BINDIR); \
+	$(INSTALL) -m 644 $(MAN) $(PREFIX)/$(MANDIR)/man`echo $(MAN) | sed "s/.*\.//"`/$(MAN); \
+	fi
 
 co:
 	co RCS/*,v
 	(cd examples; co RCS/*,v)
 
 cvstag:
-	cvs tag -F $(CVSTAG) .
+	cvs tag -cF $(CVSTAG) .
 
 archive: cvstag
 	@rm -rf /tmp/logrotate-$(VERSION) /tmp/logrotate
-	@cd /tmp; cvs export -r$(CVSTAG) logrotate; mv logrotate logrotate-$(VERSION)
+	@cd /tmp; cvs -d $(CVSROOT) export -r$(CVSTAG) logrotate; mv logrotate logrotate-$(VERSION)
 	@cd /tmp/logrotate-$(VERSION)
 	@cd /tmp; tar czSpf logrotate-$(VERSION).tar.gz logrotate-$(VERSION)
 	@rm -rf /tmp/logrotate-$(VERSION)

=== added file 'README.HPUX'
--- README.HPUX	1970-01-01 00:00:00 +0000
+++ README.HPUX	2004-06-11 13:51:34 +0000
@@ -0,0 +1,43 @@
+How to build and install logrotate on HP-UX 11.00 (these instructions should
+also work on HP-UX 10.20):
+ 
+1.  Obtain and install the following GNU packages for HP-UX:
+        binutils 2.9.1
+        gcc 2.95.2
+        make 3.78.1
+    I used the packages at the Software Porting and Archive Centre for HP-UX
+    at http://hpux.cs.utah.edu/.
+
+    Obtain and install the following GNU/Linux to HP-UX Porting package:
+	libhplx library
+    See http://devresource.hp.com/LPK/index.html for downloads.
+    This library is needed to provide the ??? function.
+ 
+2.  Obtain, build, and install popt 1.4 (there doesn't seem to be a build at
+    the Porting Centre.)
+    See ftp://ftp.rpm.org/pub/rpm/dist/rpm-4.0.x/popt-1.6.4.tar.gz
+    Install it into the directory of your choice (i.e. 
+    "./configure --prefix=/opt/popt").
+ 
+3.  Build logrotate, telling it where to find popt and hplx installation. The
+    POPT_DIR defaults to /usr/local and HPLX_DIR defaults to /usr/local/hplx:
+        gmake POPT_DIR=/usr/local HPLX_DIR=/usr/local/hplx
+ 
+4.  Install logrotate into your desired directory (BASEDIR defaults to
+    /usr/local):
+        gmake install BASEDIR=/usr/local
+ 
+5.  Copy the configuration files into your desired location.
+        cp examples/logrotate-default /etc/logrotate.conf
+	mkdir /etc/logrotate.d
+
+6.  Set up a cron job to run logrotate daily. See examples/logrotate.cron.
+ 
+7.  I also recommend setting CLEAN_ADM=0 in /etc/rc.config.d/clean, and
+    setting up an init script to use logrotate for this instead.  This way,
+    logrotate manages all logfile pruning.
+ 
+ 
+Questions, comments, abuse to:
+    Paul D. Gear <citecpdg@citec.qld.gov.au>, <paulgear@bigfoot.com>
+    Danial M. Howard <howadani@isu.edu>, <dmhoward@byu.edu>

=== added file 'README.Solaris'
--- README.Solaris	1970-01-01 00:00:00 +0000
+++ README.Solaris	2004-06-11 13:51:34 +0000
@@ -0,0 +1,19 @@
+Steps to build and install logrotate on Solaris 2.6 
+ 
+1.  Obtain and install the following GNU packages from http://Sunfreeware.com:
+        gcc 2.95.2
+        make 3.80
+	popt-1.6.3
+
+2.  Build and install logrotate:
+	gmake
+	gmake install
+ 
+OBS.: If you want to use the test script on Solaris 2.6, you'll need to have
+      bash installed, adjust the path after the sha-bang (#!) properly and
+      substitute the sintax $(...) for backticks `...` in all
+      "test-config.?.in" files.
+
+-- 
+Fidelis Assis <fidelis@embratel.net.br>
+

=== modified file 'basenames.c'
--- basenames.c	2002-04-23 23:16:43 +0000
+++ basenames.c	2004-06-11 13:51:34 +0000
@@ -23,9 +23,8 @@
 char * ourDirName(char * origname) {
     char * slash;
     char * name;
-    int i = strlen(origname);
 
-    name = strcpy(malloc(i + 1), origname);
+    name = strdup(origname);
 
     stripTrailingSlashes(name);
 

=== modified file 'config.c'
--- config.c	2002-04-23 23:16:43 +0000
+++ config.c	2004-06-11 13:51:34 +0000
@@ -13,6 +13,7 @@
 #include <sys/stat.h>
 #include <time.h>
 #include <unistd.h>
+#include <assert.h>
 
 #include "basenames.h"
 #include "log.h"
@@ -22,9 +23,13 @@
 #define GLOB_ABORTED GLOB_ABEND
 #endif
 
+#define REALLOC_STEP    10
+
+#if defined(SunOS) && !defined(isblank)
+#define isblank(c) 	( (c) == ' ' || (c) == '\t' ) ? 1 : 0
+#endif
+
 static char * defTabooExts[] = { ".rpmsave", ".rpmorig", "~", ",v",
-				 ".disabled", ".dpkg-old", ".dpkg-dist",
-				 ".dpkg-new",
 				 ".rpmnew", ".swp" };
 static int defTabooCount = sizeof(defTabooExts) / sizeof(char *);
 
@@ -35,6 +40,8 @@
 static int readConfigFile(const char * configFile, logInfo * defConfig, 
 			  logInfo ** logsPtr, int * numLogsPtr);
 static int globerr(const char * pathname, int theerr);
+static struct rotatePatternElement * parsePattern(const char * pattern,
+					const char * configFile, int lineNum);
 
 static int isolateValue(const char * fileName, int lineNum, char * key, 
 			char ** startPtr, char ** endPtr) {
@@ -127,13 +134,116 @@
 	return NULL;
 }
 
+static int checkFile(const char * fname) {
+    int i;
+
+    /* Check if fname is '.' or '..'; if so, return false */
+    if (fname[0] == '.' &&
+	(!fname[1] || (fname[1] == '.' && !fname[2])))
+	return 0;
+
+    /* Check if fname is ending in a taboo-extension; if so, return
+       false */
+    for (i = 0; i < tabooCount; i++) {
+	if (!strcmp(fname + strlen(fname) - strlen(tabooExts[i]),
+	    tabooExts[i])) {
+	    message(MESS_ERROR, "Ignoring %s, because of %s "
+		    "ending\n", fname, tabooExts[i]);
+
+	    return 0;
+	}
+    }
+
+      /* All checks have been passed; return true */
+    return 1;
+}
+
+/* Used by qsort to sort filelist */
+static int compar(const void *p, const void *q) {
+	return  strcoll(*((char **)p), *((char **)q));
+}
+
+/* Free memory blocks pointed to by pointers in namelist and namelist itself */
+static void free_namelist (char **namelist, int files_count)
+{
+    int i;
+    for (i=0; i<files_count; ++i)
+	free(namelist[i]);
+    free(namelist);
+}
+
+static struct rotatePatternElement * parsePattern(const char * pattern,
+					const char * configFile, int lineNum) {
+    struct rotatePatternElement * head, * item;
+    struct rotatePatternElement new;
+    const char * field;
+    const char * start;
+
+    /* dummy head node; we build off of it */
+    head = alloca(sizeof(*head));
+    item = head;
+
+    start = field = pattern;
+    memset(&new, 0, sizeof(new));
+    while (*field) {
+	if (*field != '%') {
+	    field++;
+	    continue;
+	}
+
+	switch (*field) {
+	  case '%':
+	    break;
+	  case 'f':
+	    new.type = RP_FILENAME;
+	    break;
+	  case 'c':
+	    new.type = RP_COUNT;
+	    break;
+	  default:
+	    message(MESS_ERROR, "%s:%d unknown element %c in pattern %s",
+		    configFile, lineNum, *field, pattern);
+	    return NULL;
+	}
+
+	if (new.type != RP_NONE) {
+	    if (start != field) {
+		/* fixed string */
+		item->next = malloc(sizeof(*item->next));
+		item->type = RP_STRING;
+		item->arg = malloc(field - start + 1);
+		strncpy(item->arg, start, field - start);
+		item->arg[field-start] = '\0';
+		item = item->next;
+	    }
+
+	    item->next = malloc(sizeof(*item->next));
+	    *item->next = new;
+	    item = item->next;
+
+	    memset(&new, 0, sizeof(new));
+	}
+
+	field++;
+    }
+
+    if (start != field) {
+	/* fixed string */
+	item->next = malloc(sizeof(*item->next));
+	item->type = RP_STRING;
+	item->arg = malloc(field - start + 1);
+	strncpy(item->arg, start, field-start);
+	item->arg[field-start] = '\0';
+	item = item->next;
+    }
+    
+    return NULL;
+}
+
 int readConfigPath(const char * path, logInfo * defConfig, 
 			  logInfo ** logsPtr, int * numLogsPtr) {
     struct stat sb;
-    DIR * dir;
-    struct dirent * ent;
     int here;
-    int i;
 
     if (!tabooExts) {
 	tabooExts = malloc(sizeof(*tabooExts) * defTabooCount);
@@ -147,67 +257,81 @@
     }
 
     if (S_ISDIR(sb.st_mode)) {
-	dir = opendir(path);
-	if (!dir) {
-	    message(MESS_ERROR, "failed to open directory %s: %s\n", path,
-			strerror(errno));
-	    return 1;
-	}
+	char		**namelist, **p;
+	struct dirent	*dp;
+	int		files_count,i;
+	DIR		*dirp;
 
 	here = open(".", O_RDONLY);
 	if (here < 0) {
 	    message(MESS_ERROR, "cannot open current directory: %s\n", 
 		    strerror(errno));
-	    closedir(dir);
-	    return 1;
+	    return 1;
+	}
+
+	if ( (dirp = opendir(path)) == NULL) {
+	    message(MESS_ERROR, "cannot open directory %s: %s\n", path,
+		    strerror(errno));
+	    return 1;
+	}
+	files_count = 0;
+	namelist = NULL;
+	while ((dp = readdir(dirp)) != NULL) {
+	    if (checkFile(dp->d_name)) {
+		/* Realloc memory for namelist array if necessary */
+		if (files_count % REALLOC_STEP == 0) {
+		    p = (char **) realloc(namelist, (files_count + REALLOC_STEP) * sizeof(char *));
+		    if (p) {
+			namelist = p;
+			memset(namelist + files_count, '\0', REALLOC_STEP * sizeof(char *));
+		    } else {
+			free_namelist(namelist, files_count);
+			message(MESS_ERROR, "cannot realloc: %s\n", strerror(errno));
+			return 1;
+		    }
+		}
+		/* Alloc memory for file name */
+		if ( (namelist[files_count] = (char *) malloc( strlen(dp->d_name) + 1)) ) {
+		    strcpy(namelist[files_count], dp->d_name);
+		    files_count++;
+		} else {
+		    free_namelist(namelist, files_count);
+		    message(MESS_ERROR, "cannot realloc: %s\n", strerror(errno));
+		    return 1;
+		}
+	    }
+	}
+	closedir( dirp );
+
+	if (files_count > 0) {
+	    qsort(namelist, files_count, sizeof(char *), compar);
+	} else {
+	    return 0;
 	}
 
 	if (chdir(path)) {
 	    message(MESS_ERROR, "error in chdir(\"%s\"): %s\n", path,
 		    strerror(errno));
 	    close(here);
-	    closedir(dir);
+	    free_namelist(namelist, files_count);
 	    return 1;
 	}
 
-	do {
-	    errno = 0;
-	    ent = readdir(dir);
-	    if (errno) {
-		message(MESS_ERROR, "readdir() from %s failed: %s\n", path,
-			strerror(errno));
-		fchdir(here);
-		close(here);
-		closedir(dir);
-		return 1;
-	    } else if (ent && ent->d_name[0] == '.' && (!ent->d_name[1] || 
-		(ent->d_name[1] == '.' && !ent->d_name[2]))) {
-		/* noop */
-	    } else if (ent) {
-		for (i = 0; i < tabooCount; i++) {
-		    if (!strcmp(ent->d_name + strlen(ent->d_name) -
-				strlen(tabooExts[i]), tabooExts[i])) {
-			message(MESS_DEBUG, "Ignoring %s, because of %s "
-				"ending\n", ent->d_name, tabooExts[i]);
-			break;
-		    }
-		}
-
-		if (i == tabooCount) {
-		    if (readConfigFile(ent->d_name, defConfig, logsPtr, 
-				       numLogsPtr)) {
-			fchdir(here);
-			close(here);
-			return 1;
-		    }
-		}
-	    }
-	} while (ent);
-
-	closedir(dir);
+	for (i=0; i<files_count; ++i) {
+	  assert(namelist[i] != NULL);
+	  
+	  if (readConfigFile(namelist[i], defConfig, logsPtr, 
+			     numLogsPtr)) {
+	    fchdir(here);
+	    close(here);
+	    free_namelist(namelist, files_count);
+	    return 1;
+	  }
+	};
 
 	fchdir(here);
 	close(here);
+	free_namelist(namelist, files_count);
     } else {
 	return readConfigFile(path, defConfig, logsPtr, numLogsPtr);
     }
@@ -217,9 +341,9 @@
 
 static int globerr(const char * pathname, int theerr) {
     message(MESS_ERROR, "error accessing %s: %s\n", pathname, 
-            strerror(theerr));
+	    strerror(theerr));
 
-    /* We want the glob operation to continue, so return 0 */
+    /* We want the glob operation to abort on error, so return 1 */
     return 1;
 }
 
@@ -371,6 +495,14 @@
 		newlog->flags &= ~LOG_FLAG_COPYTRUNCATE;
 
 		*endtag = oldchar, start = endtag;
+	    } else if (!strcmp(start, "copy")) {
+		newlog->flags |= LOG_FLAG_COPY;
+
+		*endtag = oldchar, start = endtag;
+	    } else if (!strcmp(start, "nocopy")) {
+		newlog->flags &= ~LOG_FLAG_COPY;
+
+		*endtag = oldchar, start = endtag;
 	    } else if (!strcmp(start, "ifempty")) {
 		newlog->flags |= LOG_FLAG_IFEMPTY;
 
@@ -453,6 +585,9 @@
 		    } else if (start[length] == 'M') {
 			start[length] = '\0';
 			multiplier = 1024 * 1024;
+		    } else if (start[length] == 'G') {
+			start[length] = '\0';
+			multiplier = 1024 * 1024 * 1024;
 		    } else if (!isdigit(start[length])) {
 			message(MESS_ERROR, "%s:%d unknown unit '%c'\n",
 				    configFile, lineNum, start[length]);
@@ -520,13 +655,28 @@
 		    }
 		    *endtag = oldchar, start = endtag;
 		}
+	    } else if (!strcmp(start, "start")) {
+		*endtag = oldchar, start = endtag;
+
+		if (!isolateValue(configFile, lineNum, "start count", &start,
+				  &endtag)) {
+		    oldchar = *endtag, *endtag = '\0';
+
+		    newlog->logStart = strtoul(start, &chptr, 0);
+		    if (*chptr || newlog->logStart < 0) {
+		      message(MESS_ERROR, "%s:%d bad start count '%s'\n",
+			      configFile, lineNum, start);
+		      return 1;
+		    }
+		    *endtag = oldchar, start = endtag;
+		}
 	    } else if (!strcmp(start, "errors")) {
 		message(MESS_DEBUG, "%s: %d: the errors directive is deprecated and no longer used.\n",
 			configFile, lineNum);
 	    } else if (!strcmp(start, "mail")) {
 		*endtag = oldchar, start = endtag;
 		if (!(newlog->logAddress = readAddress(configFile, lineNum, 
-						        "mail", &start))) {
+							"mail", &start))) {
 		    return 1;
 		}
 	    } else if (!strcmp(start, "nomail")) {
@@ -549,6 +699,13 @@
 		scriptDest = &newlog->pre;
 
 		while (*start != '\n') start++;
+	    } else if (!strcmp(start, "firstaction")) {
+		*endtag = oldchar, start = endtag;
+
+		scriptStart = start;
+		scriptDest = &newlog->first;
+
+		while (*start != '\n') start++;
 	    } else if (!strcmp(start, "postrotate")) {
 		*endtag = oldchar, start = endtag;
 
@@ -556,6 +713,13 @@
 		scriptDest = &newlog->post;
 
 		while (*start != '\n') start++;
+	    } else if (!strcmp(start, "lastaction")) {
+		*endtag = oldchar, start = endtag;
+
+		scriptStart = start;
+		scriptDest = &newlog->last;
+
+		while (*start != '\n') start++;
 	    } else if (!strcmp(start, "tabooext")) {
 		if (newlog != defConfig) {
 		    message(MESS_ERROR, "%s:%d tabooext may not appear inside "
@@ -564,7 +728,7 @@
 		}
 
 		*endtag = oldchar, start = endtag;
-		if (!isolateValue(configFile, lineNum, "size", &start, 
+		if (!isolateValue(configFile, lineNum, "tabooext", &start, 
 				  &endtag)) {
 		    oldchar = *endtag, *endtag = '\0';
 
@@ -579,7 +743,7 @@
 
 		    while (*start) {
 			chptr = start;
-			while (!isspace(*chptr) && /* *chptr != ',' && */ *chptr)
+			while (!isspace(*chptr) && *chptr != ',' && *chptr)
 			    chptr++;
 
 			tabooExts = realloc(tabooExts, sizeof(*tabooExts) * 
@@ -591,7 +755,7 @@
 			tabooCount++;
 
 			start = chptr;
-			/* if (*start == ',') start++; */
+			if (*start == ',') start++;
 			while (isspace(*start) && *start) start++;
 		    }
 
@@ -605,7 +769,7 @@
 		}
 
 		*endtag = oldchar, start = endtag;
-		if (!isolateValue(configFile, lineNum, "size", &start, 
+		if (!isolateValue(configFile, lineNum, "include", &start, 
 				  &endtag)) {
 		    oldchar = *endtag, *endtag = '\0';
 
@@ -617,6 +781,20 @@
 
 		    *endtag = oldchar, start = endtag;
 		}
+	    } else if (!strcmp(start, "pattern")) {
+		char * patternString;
+
+		*endtag = oldchar, start = endtag;
+		if (!(patternString = readPath(configFile, lineNum,
+						"pattern", &start))) {
+		    return 1;
+		}
+
+		newlog->rotatePattern = parsePattern(patternString,
+					    configFile, lineNum);
+		if (!newlog->rotatePattern) return 1;
+
+		message(MESS_DEBUG, "pattern is now %s\n", patternString);
 	    } else if (!strcmp(start, "olddir")) {
 		*endtag = oldchar, start = endtag;
 		if (!(newlog->oldDir = readPath(configFile, lineNum,
@@ -624,6 +802,7 @@
 		    return 1;
 		}
 
+#if 0
 		if (stat(newlog->oldDir, &sb)) {
 		    message(MESS_ERROR, "%s:%d error verifying olddir "
 				"path %s: %s\n", configFile, lineNum, 
@@ -637,6 +816,7 @@
 				newlog->oldDir);
 		    return 1;
 		}
+#endif
 
 		message(MESS_DEBUG, "olddir is now %s\n", newlog->oldDir);
 	    } else if (!strcmp(start, "extension")) {
@@ -682,13 +862,22 @@
 		message(MESS_DEBUG, "uncompress_prog is now %s\n", newlog->uncompress_prog);
 
 	    } else if (!strcmp(start, "compressoptions")) {
+		char * options;
+
 		*endtag = oldchar, start = endtag;
-		if (!(newlog->compress_options = readPath(configFile, lineNum, "compressoptions", &start))) {
-		    return 1;
-		}
-
-		message(MESS_DEBUG, "compress_options is now %s\n", newlog->compress_options);
-
+		if (!(options = readPath(configFile, lineNum, "compressoptions", &start))) {
+		    return 1;
+		}
+
+		if (poptParseArgvString(options, 
+					&newlog->compress_options_count,
+					&newlog->compress_options_list)) {
+		    message(MESS_ERROR, "%s:%d invalid compression options\n", 
+			    configFile, lineNum);
+		    return 1;
+		}
+
+		message(MESS_DEBUG, "compress_options is now %s\n", options);
 	    } else if (!strcmp(start, "compressext")) {
 		*endtag = oldchar, start = endtag;
 		if (!(newlog->compress_ext = readPath(configFile, lineNum, "compress-ext", &start))) {
@@ -749,6 +938,9 @@
 	    for (argNum = 0; argNum < argc; argNum++) {
 		rc = glob(argv[argNum], GLOB_NOCHECK, globerr, &globResult);
 		if (rc == GLOB_ABORTED) {
+		    if(newlog->flags & LOG_FLAG_MISSINGOK)
+		        continue;
+
 		    message(MESS_ERROR, "%s:%d glob failed for %s\n",
 			    configFile, lineNum, argv[argNum]);
 		    return 1;
@@ -796,14 +988,8 @@
 	    }
 
 	    if (newlog->oldDir) {
-		if (stat(newlog->oldDir, &sb)) {
-		    message(MESS_ERROR, "%s:%d error verifying olddir "
-				"path %s: %s\n", configFile, lineNum, 
-				newlog->oldDir, strerror(errno));
-		    return 1;
-		}
-
 		for (i = 0; i < newlog->numFiles; i++) {
+		    char *ld;
 		    dirName = ourDirName(newlog->files[i]);
 		    if (stat(dirName, &sb2)) {
 			message(MESS_ERROR, "%s:%d error verifying log file "
@@ -812,9 +998,19 @@
 			free(dirName);
 			return 1;
 		    }
-
+		    ld = alloca(strlen(dirName) + strlen(newlog->oldDir) + 2);
+		    sprintf(ld, "%s/%s", dirName, newlog->oldDir);
 		    free(dirName);
 
+		    if(newlog->oldDir[0] != '/') dirName = ld;
+		    else dirName = newlog->oldDir;
+		    if(stat(dirName, &sb)) {
+			message(MESS_ERROR, "%s:%d error verifying olddir "
+				"path %s: %s\n", configFile, lineNum,
+				dirName, strerror(errno));
+				return 1;
+		    }
+
 		    if (sb.st_dev != sb2.st_dev) {
 			message(MESS_ERROR, "%s:%d olddir %s and log file %s "
 				    "are on different devices\n", configFile,
@@ -844,6 +1040,12 @@
 	}
     }
 
+    if(scriptStart) {
+      message(MESS_ERROR, "%s:prerotate or postrotate without endscript\n",
+	      configFile);
+      return 1;
+    }
+
     return 0;
 }
 

=== added file 'config.h'
--- config.h	1970-01-01 00:00:00 +0000
+++ config.h	2004-06-11 13:51:34 +0000
@@ -0,0 +1,41 @@
+/*
+ * OS-specific definitions
+ */
+
+#ifdef __hpux
+    #define DEFAULT_MAIL_COMMAND "/usr/bin/mailx -s"
+    #define COMPRESS_COMMAND "/usr/contrib/bin/gzip"
+    #define UNCOMPRESS_COMMAND " /usr/contrib/bin/gunzip"
+    #define STATEFILE "/var/run/logrotate.status"
+#endif
+
+#ifdef SunOS
+    #define DEFAULT_MAIL_COMMAND "/usr/bin/mailx -s"
+    #define COMPRESS_COMMAND "/usr/local/bin/gzip"
+    #define UNCOMPRESS_COMMAND "/usr/local/bin/gunzip"
+    #define STATEFILE "/var/log/logrotate.status"
+#endif
+
+/*
+ * Default settings for Linux - leave these last.
+ */
+#ifndef DEFAULT_MAIL_COMMAND
+    #define DEFAULT_MAIL_COMMAND "/bin/mail -s"
+#endif
+
+#ifndef COMPRESS_COMMAND
+    #define COMPRESS_COMMAND "/bin/gzip"
+#endif
+
+#ifndef COMPRESS_EXT
+    #define COMPRESS_EXT ".gz"
+#endif
+
+#ifndef UNCOMPRESS_COMMAND
+    #define UNCOMPRESS_COMMAND "/bin/gunzip"
+#endif
+
+#ifndef STATEFILE
+    #define STATEFILE "/var/lib/logrotate.status"
+#endif
+

=== modified file 'debian/changelog'
--- debian/changelog	2002-04-23 23:16:43 +0000
+++ debian/changelog	2004-06-11 13:51:34 +0000
@@ -1,3 +1,90 @@
+logrotate (3.7-2) unstable; urgency=high
+
+  * Added commented out stuff in debian/rules to build a 
+    logrotate-selinux package should Russell's move to get libselinux1 
+    made "base" fail.
+  * Patch: 21-taboo-to-debug, reduces the "Ignoring..." messages of
+    the taboo filter from ERROR to DEBUG. (Closes: #249073)
+  * Patch: 30-config-h, changed to fix upstream bug with mailing
+    logs. (Closes: #253837)
+
+ -- Paul Martin <pm@debian.org>  Fri, 11 Jun 2004 13:51:34 +0100
+
+logrotate (3.7-1) unstable; urgency=low
+
+  * Moved to using dpatch to keep track of changes.
+  * New upstream release:
+    + Includes some SELinux code, not compiled in by default as
+      libselinux1 is not a base package. (Refer: #224880)
+    + Always use compressext for the extension for compressed
+      files; before compresscmd and compressext had to agree
+    + Compression and scripts don't use system() anymore
+    + compress and maillast didn't work together properly
+    + delaycompress and mailfirst didn't work properly
+      (Closes: #186818,#211687)
+    + Don't use system() for mailing (or uncompressing) logs anymore
+    + missingok works on directories, too (Closes: #246352,#248318)
+  * Manpage fixes:
+    + Include mention of G suffix to "size" option.
+    + Fix also the wording of the "weekly" option. (Closes: #232263)
+    + Fix examples' location of killall. (Closes: #189243)
+    + Fix description of "compressext". (Closes: #230519)
+  * Patches applied:
+    + Trustix logrotate-3.7-compressowner.patch, which retains
+      original owner and mode of logs when they are compressed.
+  * Fixed upstream source location in copyright file. (Closes: #187384)
+  * Standards-Version: 3.6.1
+    + New default compilation options (10.1).
+  * Use and depend on debhelper 4.
+  * Added reportbug script.
+
+ -- Paul Martin <pm@debian.org>  Mon, 10 May 2004 19:43:54 +0100
+
+logrotate (3.6.5-3) unstable; urgency=low
+
+  * Added missingok to /var/log/wtmp entry of logrotate.conf.
+    (Closes: #199154)
+
+ -- Paul Martin <pm@debian.org>  Tue, 11 Nov 2003 23:17:24 +0000
+
+logrotate (3.6.5-2) unstable; urgency=low
+
+  * Added check for prerotate and postrotate scripts having no 
+    terminating endscript. (Closes: #162976)
+
+ -- Paul Martin <pm@debian.org>  Wed,  9 Oct 2002 18:21:36 +0100
+
+logrotate (3.6.5-1) unstable; urgency=low
+
+  * New upstream release
+    - Stop the shared postrotate/prerotate scripts from running if none
+      of the log(s) need rotating (Closes: #149135, #153284, #15890)
+    - Use TMPDIR instead of /tmp, if specified, when executing scripts.
+    - New config directives "copy", "nocopy" and "start".
+    - Sizes can be expressed in gigabytes (G suffix).
+    - Some updates to manpage.
+       + Manpage documents naming of rotated files (Closes: #143169)
+  * Standards-Version: 3.5.6
+
+ -- Paul Martin <pm@debian.org>  Tue, 27 Aug 2002 00:21:14 +0100
+
+logrotate (3.5.9-10) unstable; urgency=high
+
+  * cron.daily was getting deleted by the debian/rules script. Mea
+    maxima culpa. (Closes: #152966).
+
+ -- Paul Martin <pm@debian.org>  Tue, 30 Jul 2002 00:08:37 +0100
+
+logrotate (3.5.9-9) unstable; urgency=high
+
+  * Downgraded depends on mailx to a recommends.
+    (Closes: #127239, #149535)
+  * Really put check for existence of logrotate binary into the
+    cron.daily script. It was getting overwritten by a copy from the
+    examples directory. (Closes: #150727)
+
+ -- Paul Martin <pm@debian.org>  Sat, 22 Jun 2002 23:48:27 +0100
+
 logrotate (3.5.9-8) unstable; urgency=high
 
   * Added check to cron.daily (blatantly stolen from exim's) to only

=== added file 'debian/compat'
--- debian/compat	1970-01-01 00:00:00 +0000
+++ debian/compat	2004-06-11 13:51:34 +0000
@@ -0,0 +1,1 @@
+4

=== modified file 'debian/control'
--- debian/control	2002-04-23 23:16:43 +0000
+++ debian/control	2004-06-11 13:51:34 +0000
@@ -2,12 +2,13 @@
 Section: admin
 Priority: important
 Maintainer: Paul Martin <pm@debian.org>
-Build-Depends: libpopt-dev, debhelper (>> 3.0.0)
-Standards-Version: 3.5.2
+Build-Depends: libpopt-dev, debhelper (>= 4), dpatch
+Standards-Version: 3.6.1
 
 Package: logrotate
 Architecture: any
-Depends: ${shlibs:Depends}, cron (>= 3.0pl1-53) | anacron, mailx, base-passwd (>= 2.0.3.4)
+Depends: ${shlibs:Depends}, ${misc:Depends}, cron (>= 3.0pl1-53) | anacron, base-passwd (>= 2.0.3.4)
+Recommends: mailx
 Description: Log rotation utility
  The logrotate utility is designed to simplify the administration of
  log files on a system which generates a lot of log files.  Logrotate

=== added file 'debian/cron.daily'
--- debian/cron.daily	1970-01-01 00:00:00 +0000
+++ debian/cron.daily	2004-06-11 13:51:34 +0000
@@ -0,0 +1,4 @@
+#!/bin/sh
+
+test -x /usr/sbin/logrotate || exit 0
+/usr/sbin/logrotate /etc/logrotate.conf

=== added file 'debian/logrotate.bug'
--- debian/logrotate.bug	1970-01-01 00:00:00 +0000
+++ debian/logrotate.bug	2004-06-11 13:51:34 +0000
@@ -0,0 +1,5 @@
+#!/bin/sh
+
+printf "Contents of /etc/logrotate.d\n" >&3
+ls -l /etc/logrotate.d >&3
+printf "\n" >&3

=== modified file 'debian/logrotate.conf'
--- debian/logrotate.conf	2002-04-23 23:16:43 +0000
+++ debian/logrotate.conf	2004-06-11 13:51:34 +0000
@@ -16,6 +16,7 @@
 
 # no packages own wtmp, or btmp -- we'll rotate them here
 /var/log/wtmp {
+    missingok
     monthly
     create 0664 root utmp
     rotate 1

=== modified file 'debian/logrotate.copyright'
--- debian/logrotate.copyright	2002-04-23 23:16:43 +0000
+++ debian/logrotate.copyright	2004-06-11 13:51:34 +0000
@@ -3,10 +3,12 @@
 
 This version was downloaded from RedHat's CVS server using:
 
- cvs -z9 -d :pserver:anoncvs@sources.redhat.com:/cvs/rhl login
+ cvs -z9 -d :pserver:anonymous@i18n.redhat.com:/cvs/rhl login
  # password is anoncvs
- cvs -z9 -d :pserver:anoncvs@sources.redhat.com:/cvs/rhl co logrotate
+ cvs -z9 -d :pserver:anonymous@i18n.redhat.com:/cvs/rhl co logrotate
 
 Upstream Authors are various RedHat people.
 
-Copyright: GPL
+Copyright: GNU GENERAL PUBLIC LICENSE (GPL)
+
+On a Debian system, the GPL can be found in /usr/share/common-licenses/GPL

=== modified file 'debian/logrotate.dirs'
--- debian/logrotate.dirs	2002-04-23 23:16:43 +0000
+++ debian/logrotate.dirs	2004-06-11 13:51:34 +0000
@@ -1,4 +1,5 @@
 usr/sbin
 usr/share/man/man8
+usr/share/bug/logrotate
 var/lib/logrotate
 etc/logrotate.d

=== added directory 'debian/patches'
=== added file 'debian/patches/00list'
--- debian/patches/00list	1970-01-01 00:00:00 +0000
+++ debian/patches/00list	2004-06-11 13:51:34 +0000
@@ -0,0 +1,25 @@
+# Fix the taboo list to avoid .dpkg-old, etc.
+10-taboos
+
+# Loosens the config syntax checking to allow more flexibility.
+#  (Closes: #95617,#95630)
+20-configparse
+
+# Quietens the taboo list from ERROR to a DEBUG
+#  (Closes: #249073)
+21-taboo-to-debug
+
+# Fix locations of STATEFILE and DEFAULT_MAIL_COMMAND
+30-config-h
+
+# No longer consider state file dates before 1996 as fatal errors.
+# (Closes: #65534)
+40-datehack
+
+# Manpage fixes.
+50-manpage
+51-man-189243
+
+# Security patch (from Trustix)
+#  retains owner and mode of compressed logs
+60-compressowner

=== added file 'debian/patches/10-taboos.dpatch'
--- debian/patches/10-taboos.dpatch	1970-01-01 00:00:00 +0000
+++ debian/patches/10-taboos.dpatch	2004-06-11 13:51:34 +0000
@@ -0,0 +1,37 @@
+#!/bin/sh -e
+## 02-taboos.dpatch by Paul Martin <pm@debian.org>
+##
+## All lines beginning with `## DP:' are a description of the patch.
+## DP: No description.
+
+if [ $# -lt 1 ]; then
+    echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+    exit 1
+fi
+
+[ -f debian/patches/00patch-opts ] && . debian/patches/00patch-opts
+patch_opts="${patch_opts:--f --no-backup-if-mismatch}"
+
+case "$1" in
+       -patch) patch $patch_opts -p1 < $0;;
+       -unpatch) patch $patch_opts -p1 -R < $0;;
+        *)
+                echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+                exit 1;;
+esac
+
+exit 0
+
+@DPATCH@
+diff -urNad /home/pm/deb/logrotate/logrotate-3.7/config.c logrotate-3.7/config.c
+--- /home/pm/deb/logrotate/logrotate-3.7/config.c	2003-09-22 20:11:12.000000000 +0100
++++ logrotate-3.7/config.c	2004-02-11 18:16:09.000000000 +0000
+@@ -30,6 +30,8 @@
+ #endif
+ 
+ static char * defTabooExts[] = { ".rpmsave", ".rpmorig", "~", ",v",
++				 ".disabled", ".dpkg-old", ".dpkg-dist",
++				 ".dpkg-new",
+ 				 ".rpmnew", ".swp" };
+ static int defTabooCount = sizeof(defTabooExts) / sizeof(char *);
+ 

=== added file 'debian/patches/20-configparse.dpatch'
--- debian/patches/20-configparse.dpatch	1970-01-01 00:00:00 +0000
+++ debian/patches/20-configparse.dpatch	2004-06-11 13:51:34 +0000
@@ -0,0 +1,46 @@
+#!/bin/sh -e
+## 03-configparse.dpatch by Paul Martin <pm@debian.org>
+##
+## All lines beginning with `## DP:' are a description of the patch.
+## DP: No description.
+
+if [ $# -lt 1 ]; then
+    echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+    exit 1
+fi
+
+[ -f debian/patches/00patch-opts ] && . debian/patches/00patch-opts
+patch_opts="${patch_opts:--f --no-backup-if-mismatch}"
+
+case "$1" in
+       -patch) patch $patch_opts -p1 < $0;;
+       -unpatch) patch $patch_opts -p1 -R < $0;;
+        *)
+                echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+                exit 1;;
+esac
+
+exit 0
+
+@DPATCH@
+diff -urNad /home/pm/deb/logrotate/logrotate-3.7/config.c logrotate-3.7/config.c
+--- /home/pm/deb/logrotate/logrotate-3.7/config.c	2004-02-11 18:17:16.000000000 +0000
++++ logrotate-3.7/config.c	2004-02-11 18:19:12.000000000 +0000
+@@ -745,7 +745,7 @@
+ 
+ 		    while (*start) {
+ 			chptr = start;
+-			while (!isspace(*chptr) && *chptr != ',' && *chptr)
++			while (!isspace(*chptr) && /* *chptr != ',' && */ *chptr)
+ 			    chptr++;
+ 
+ 			tabooExts = realloc(tabooExts, sizeof(*tabooExts) * 
+@@ -757,7 +757,7 @@
+ 			tabooCount++;
+ 
+ 			start = chptr;
+-			if (*start == ',') start++;
++			/* if (*start == ',') start++; */
+ 			while (isspace(*start) && *start) start++;
+ 		    }
+ 

=== added file 'debian/patches/21-taboo-to-debug.dpatch'
--- debian/patches/21-taboo-to-debug.dpatch	1970-01-01 00:00:00 +0000
+++ debian/patches/21-taboo-to-debug.dpatch	2004-06-11 13:51:34 +0000
@@ -0,0 +1,37 @@
+#! /bin/sh -e
+## 21-taboo-to-debug.dpatch by Paul Martin <pm@debian.org>
+##
+## All lines beginning with `## DP:' are a description of the patch.
+## DP: No description.
+
+if [ $# -lt 1 ]; then
+    echo "`basename $0`: script expects -patch|-unpatch as argument" >&2
+    exit 1
+fi
+
+[ -f debian/patches/00patch-opts ] && . debian/patches/00patch-opts
+patch_opts="${patch_opts:--f --no-backup-if-mismatch} ${2:+-d $2}"
+
+case "$1" in
+    -patch) patch -p1 ${patch_opts} < $0;;
+    -unpatch) patch -R -p1 ${patch_opts} < $0;;
+    *)
+        echo "`basename $0`: script expects -patch|-unpatch as argument" >&2
+        exit 1;;
+esac
+
+exit 0
+
+@DPATCH@
+diff -urNad /home/pm/deb/logrotate/logrotate-3.7/config.c logrotate-3.7/config.c
+--- /home/pm/deb/logrotate/logrotate-3.7/config.c	2004-05-15 00:29:38.000000000 +0100
++++ logrotate-3.7/config.c	2004-05-15 00:29:57.000000000 +0100
+@@ -149,7 +149,7 @@
+     for (i = 0; i < tabooCount; i++) {
+ 	if (!strcmp(fname + strlen(fname) - strlen(tabooExts[i]),
+ 	    tabooExts[i])) {
+-	    message(MESS_ERROR, "Ignoring %s, because of %s "
++	    message(MESS_DEBUG, "Ignoring %s, because of %s "
+ 		    "ending\n", fname, tabooExts[i]);
+ 
+ 	    return 0;

=== added file 'debian/patches/30-config-h.dpatch'
--- debian/patches/30-config-h.dpatch	1970-01-01 00:00:00 +0000
+++ debian/patches/30-config-h.dpatch	2004-06-11 13:51:34 +0000
@@ -0,0 +1,45 @@
+#!/bin/sh -e
+## 30-config-h.dpatch by Paul Martin <pm@debian.org>
+##
+## All lines beginning with `## DP:' are a description of the patch.
+## DP: No description.
+
+if [ $# -lt 1 ]; then
+    echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+    exit 1
+fi
+
+[ -f debian/patches/00patch-opts ] && . debian/patches/00patch-opts
+patch_opts="${patch_opts:--f --no-backup-if-mismatch}"
+
+case "$1" in
+       -patch) patch $patch_opts -p1 < $0;;
+       -unpatch) patch $patch_opts -p1 -R < $0;;
+        *)
+                echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+                exit 1;;
+esac
+
+exit 0
+
+@DPATCH@
+diff -urNad /home/pm/deb/logrotate/logrotate-3.7/config.h logrotate-3.7/config.h
+--- /home/pm/deb/logrotate/logrotate-3.7/config.h	2004-06-11 13:48:48.000000000 +0100
++++ logrotate-3.7/config.h	2004-06-11 13:51:03.000000000 +0100
+@@ -20,7 +20,7 @@
+  * Default settings for Linux - leave these last.
+  */
+ #ifndef DEFAULT_MAIL_COMMAND
+-    #define DEFAULT_MAIL_COMMAND "/bin/mail -s"
++    #define DEFAULT_MAIL_COMMAND "/usr/bin/mail"
+ #endif
+ 
+ #ifndef COMPRESS_COMMAND
+@@ -36,6 +36,6 @@
+ #endif
+ 
+ #ifndef STATEFILE
+-    #define STATEFILE "/var/lib/logrotate.status"
++    #define STATEFILE "/var/lib/logrotate/status"
+ #endif
+ 

=== added file 'debian/patches/40-datehack.dpatch'
--- debian/patches/40-datehack.dpatch	1970-01-01 00:00:00 +0000
+++ debian/patches/40-datehack.dpatch	2004-06-11 13:51:34 +0000
@@ -0,0 +1,37 @@
+#!/bin/sh -e
+## 40-datehack.dpatch by Paul Martin <pm@debian.org>
+##
+## All lines beginning with `## DP:' are a description of the patch.
+## DP: No description.
+
+if [ $# -lt 1 ]; then
+    echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+    exit 1
+fi
+
+[ -f debian/patches/00patch-opts ] && . debian/patches/00patch-opts
+patch_opts="${patch_opts:--f --no-backup-if-mismatch}"
+
+case "$1" in
+       -patch) patch $patch_opts -p1 < $0;;
+       -unpatch) patch $patch_opts -p1 -R < $0;;
+        *)
+                echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+                exit 1;;
+esac
+
+exit 0
+
+@DPATCH@
+diff -urNad /home/pm/deb/logrotate/logrotate-3.7/logrotate.c logrotate-3.7/logrotate.c
+--- /home/pm/deb/logrotate/logrotate-3.7/logrotate.c	2004-02-11 18:34:51.000000000 +0000
++++ logrotate-3.7/logrotate.c	2004-02-11 18:43:28.000000000 +0000
+@@ -981,7 +981,7 @@
+ 	}
+ 	
+ 	/* Hack to hide earlier bug */
+-	if ((year != 1900) && (year < 1996 || year > 2100)) {
++	if ((year != 1900) && (year < 1970 || year > 2100)) {
+ 	    message(MESS_ERROR, "bad year %d for file %s in state file %s\n",
+ 		    year, argv[0], stateFilename);
+ 	    fclose(f);

=== added file 'debian/patches/50-manpage.dpatch'
--- debian/patches/50-manpage.dpatch	1970-01-01 00:00:00 +0000
+++ debian/patches/50-manpage.dpatch	2004-06-11 13:51:34 +0000
@@ -0,0 +1,410 @@
+#!/bin/sh -e
+## 50-manpage.dpatch by Paul Martin <pm@debian.org>
+##
+## All lines beginning with `## DP:' are a description of the patch.
+## DP: No description.
+
+if [ $# -lt 1 ]; then
+    echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+    exit 1
+fi
+
+[ -f debian/patches/00patch-opts ] && . debian/patches/00patch-opts
+patch_opts="${patch_opts:--f --no-backup-if-mismatch}"
+
+case "$1" in
+       -patch) patch $patch_opts -p1 < $0;;
+       -unpatch) patch $patch_opts -p1 -R < $0;;
+        *)
+                echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+                exit 1;;
+esac
+
+exit 0
+
+@DPATCH@
+diff -urNad /home/pm/deb/logrotate/logrotate-3.7/logrotate.8 logrotate-3.7/logrotate.8
+--- /home/pm/deb/logrotate/logrotate-3.7/logrotate.8	2003-09-22 20:11:12.000000000 +0100
++++ logrotate-3.7/logrotate.8	2004-02-11 21:03:50.000000000 +0000
+@@ -3,7 +3,8 @@
+ .SH NAME
+ logrotate \- rotates, compresses, and mails system logs
+ .SH SYNOPSIS
+-\fBlogrotate\fR [-dv] [-f|--force] [-s|--state \fIfile\fR] \fIconfig_file\fR+
++\fBlogrotate\fR [\fB\-dv\fR] [\fB\-f\fR|\fB\-\-force\fR]
++[\fB\-s\fR|\fB-\-state\ \fIstatefile\fR] \fIconfig_file\fR ..
+ .SH DESCRIPTION
+ \fBlogrotate\fR is designed to ease administration of systems that generate
+ large numbers of log files.  It allows automatic rotation, compression, 
+@@ -11,16 +12,16 @@
+ weekly, monthly, or when it grows too large.
+ .P
+ Normally, \fBlogrotate\fR is run as a daily cron job.  It will not modify
+-a log multiple times in one day unless the criterium for that log is
+-based on the log's size and \fBlogrotate\fR is being run multiple times
+-each day, or unless the \fB-f\fR or \fB-force\fR option is used. 
++a log more than once in one day unless the criterion for that log is
++based on the log's size and \fBlogrotate\fR is being run more than once
++each day, or unless the \fB-f\fR or \fB-\-force\fR option is used. 
+ .P
+ Any number of config files may be given on the command line. Later config
+ files may override the options given in earlier files, so the order
+-in which the \fBlogrotate\fR config files are listed in is important.
++in which the \fBlogrotate\fR config files are listed is important.
+ Normally, a single config file which includes any other config files
+ which are needed should be used.  See below for more information on how
+-to use the \fIinclude\fR directive to accomplish this.  If a directory
++to use the \fBinclude\fR directive to accomplish this.  If a directory
+ is given on the command line, every file in that directory is used as
+ a config file.
+ .P
+@@ -36,37 +37,42 @@
+ be made to the logs or to the \fBlogrotate\fR state file.
+ 
+ .TP
+-\fB-f, -\-force\fR
++\fB-f\fR, \fB-\-force\fR
+ Tells \fBlogrotate\fR to force the rotation, even if it doesn't think
+ this is necessary.  Sometimes this is useful after adding new entries to
+-\fBlogrotate\fR, or if old log files have been removed by hand, as the
+-new files will be created, and logging will continue correctly.
++a \fBlogrotate\fR config file, or if old log files have been removed
++by hand, as the new files will be created, and logging will continue
++correctly.
+ 
+ .TP
+-\fB-m, -\-mail <command>\fR
++\fB-m\R, \B-\-mail <command>\fR
+ Tells \fBlogrotate\fR which command to use when mailing logs. This
+ command should accept two arguments: 1) the subject of the message, and
+ 2) the recipient. The command must then read a message on standard input
+-and mail it to the recipient. The default mail command is \fB/bin/mail
++and mail it to the recipient. The default mail command is \fB/usr/bin/mail
+ -s\fR.
+ 
+ .TP
+-\fB-s, -\-state <statefile>\fR
++\fB-s\fR, \fB-\-state <statefile>\fR
+ Tells \fBlogrotate\fR to use an alternate state file.  This is useful
+-if logrotate is being run as a different user for various sets of
++if \fBlogrotate\fR is being run as a different user for various sets of
+ log files.  The default state file is \fI/var/lib/logrotate/status\fR.
+ 
+ .TP
+ \fB-\-usage\fR
+ Prints a short usage message.
+ 
++.TP
++\fB-v\fR, \fB--verbose\fR
++Display messages during rotation.
++
+ .SH CONFIGURATION FILE
+ 
+ \fBlogrotate\fR reads everything about the log files it should be handling
+ from the series of configuration files specified on the command line.  Each
+ configuration file can set global options (local definitions override
+ global ones, and later definitions override earlier ones) and specify
+-a logfile to rotate. A simple configuration file looks like this:
++some logfiles to rotate. A simple configuration file looks like this:
+ 
+ .nf
+ .ta +3i
+@@ -77,7 +83,7 @@
+     rotate 5
+     weekly
+     postrotate
+-	/sbin/killall -HUP syslogd
++        /sbin/killall -HUP syslogd
+     endscript
+ }
+ 
+@@ -87,7 +93,7 @@
+     size=100k
+     sharedscripts
+     postrotate
+-	/sbin/killall -HUP httpd
++        /sbin/killall -HUP httpd
+     endscript
+ }
+ 
+@@ -97,7 +103,7 @@
+     olddir /var/log/news/old
+     missingok
+     postrotate
+-	kill -HUP `cat /var/run/inn.pid`
++        kill -HUP `cat /var/run/inn.pid`
+     endscript
+     nocompress
+ }
+@@ -107,9 +113,9 @@
+ The first few lines set global options; in the example, logs are
+ compressed after they are rotated.  Note that comments may appear
+ anywhere in the config file as long as the first non-whitespace
+-character on the line is a #.
++character on the line is a \fB#\fR.
+ 
+-The next section of the config files defined how to handle the log file
++The next section of the config file defines how to handle the log file
+ \fI/var/log/messages\fR. The log will go through five weekly rotations before
+ being removed. After the log file has been rotated (but before the old
+ version of the log has been compressed), the command 
+@@ -117,13 +123,13 @@
+ 
+ The next section defines the parameters for both
+ \fI/var/log/httpd/access.log\fR and \fI/var/log/httpd/error.log\fR.
+-They are rotated whenever is grows over 100k is size, and the old logs
++Each is rotated whenever it grows over 100k is size, and the old log
+ files are mailed (uncompressed) to www@my.org after going through 5
+ rotations, rather then being removed. The \fBsharedscripts\fR means that
+ the \fBpostrotate\fR script will only be run once, not once for each
+-log which is rotated. Note that the double quotes around the first filename
+-at the beginning of this section allows logrotate to rotate logs with
+-spaces in the name. Normal shell quoting rules apply, with ', ", and \\
++log which is rotated. Note that log file names may be enclosed in
++quotes (and that quotes are required if the name contains spaces).
++Normal shell quoting rules apply, with \fB'\fR, \fB"\fR, and \fB\\\fR
+ characters supported.
+ 
+ The last section defines the parameters for all of the files in
+@@ -135,29 +141,33 @@
+ rotate all files, including previously rotated ones.  A way around this
+ is to use the \fBolddir\fR directive or a more exact wildcard (such as *.log).
+ 
++If the directory \fI/var/log/news\fR does not exist, this will cause 
++\fBlogrotate\fR to report an error. This error cannot be stopped with
++the \fBmissingok\fR directive.
++
+ Here is more information on the directives which may be included in
+ a \fBlogrotate\fR configuration file:
+ 
+ .TP
+ \fBcompress\fR
+-Old versions of log files are compressed with \fBgzip\fR by default. See also
+-\fBnocompress\fR. 
++Old versions of log files are compressed with \fBgzip\fR(1) by default.
++See also \fBnocompress\fR. 
+ 
+ .TP
+ \fBcompresscmd\fR
+ Specifies which command to use to compress log files.  The default is
+-\fBgzip\fR.  See also \fBcompress\fR.
++\fBgzip\fR(1).  See also \fBcompress\fR.
+ 
+ .TP
+ \fBuncompresscmd\fR
+ Specifies which command to use to uncompress log files.  The default is
+-\fBgunzip\fR.
++\fBgunzip\fR(1).
+ 
+ .TP
+ \fBcompressext\fR
+ Specifies which extension to use on compressed logfiles, if compression
+-is enabled.  The default follows that of the configured compression
+-command.
++is enabled.  The default follows that of the default compression
++command (.gz).
+ 
+ .TP
+ \fBcompressoptions\fR
+@@ -174,9 +184,9 @@
+ 
+ .TP
+ \fBcopytruncate\fR
+-Truncate the original log file in place after creating a copy,
+-instead of moving the old log file and optionally creating a new one,
+-It can be used when some program can not be told to close its logfile
++Truncate the original log file to zero size in place after creating a copy,
++instead of moving the old log file and optionally creating a new one.
++It can be used when some program cannot be told to close its logfile
+ and thus might continue writing (appending) to the previous log file forever.
+ Note that there is a very small time slice between copying the file and
+ truncating it, so some logging data might be lost.
+@@ -188,7 +198,7 @@
+ Immediately after rotation (before the \fBpostrotate\fR script is run)
+ the log file is created (with the same name as the log file just rotated).
+ \fImode\fR specifies the mode for the log file in octal (the same
+-as \fBchmod(2)\fR), \fIowner\fR specifies the user name who will own the
++as \fBchmod\fR)(2), \fIowner\fR specifies the user name who will own the
+ log file, and \fIgroup\fR specifies the group the log file will belong
+ to. Any of the log file attributes may be omitted, in which case those
+ attributes for the new file will use the same values as the original log
+@@ -202,20 +212,20 @@
+ .TP
+ \fBdelaycompress\fR
+ Postpone compression of the previous log file to the next rotation cycle.
+-This has only effect when used in combination with \fBcompress\fR.
+-It can be used when some program can not be told to close its logfile
++This only has effect when used in combination with \fBcompress\fR.
++It can be used when some program cannot be told to close its logfile
+ and thus might continue writing to the previous log file for some time.
+ 
+ .TP
+ \fBextension \fIext\fR
+ Log files are given the final extension \fIext\fR after rotation. If 
+-compression is used, the compression extension (normally \fB.gz\fR)
++compression is used, the compression extension (normally \fI.gz\fR)
+ appears after \fIext\fR.
+ 
+ .TP
+ \fBifempty\fR
+-Rotate the log file even if it is empty, overiding the \fBnotifempty\fR
+-option (ifempty is the default).
++Rotate the log file even if it is empty, overriding the \fBnotifempty\fR
++option (\fBifempty\fR is the default).
+ 
+ .TP
+ \fBinclude \fIfile_or_directory\fR
+@@ -226,12 +236,12 @@
+ which are ignored are files which are not regular files (such as
+ directories and named pipes) and files whose names end with one of
+ the taboo extensions, as specified by the \fBtabooext\fR directive.
+-The \fBinclude\fR directive may not appear inside of a log file
++The \fBinclude\fR directive may not appear inside a log file
+ definition.
+ 
+ .TP
+ \fBmail \fIaddress\fR
+-When a log is rotated out-of-existence, it is mailed to \fIaddress\fR. If
++When a log is rotated out of existence, it is mailed to \fIaddress\fR. If
+ no mail should be generated by a particular log, the \fBnomail\fR directive
+ may be used.
+ 
+@@ -257,8 +267,7 @@
+ 
+ .TP
+ \fBnocompress\fR
+-Old versions of log files are not compressed with \fBgzip\fR. See also
+-\fBcompress\fR. 
++Old versions of log files are not compressed. See also \fBcompress\fR. 
+ 
+ .TP
+ \fBnocopy\fR
+@@ -281,7 +290,7 @@
+ 
+ .TP
+ \fBnomail\fR
+-Don't mail old log files to any address.
++Do not mail old log files to any address.
+ 
+ .TP
+ \fBnomissingok\fR
+@@ -289,12 +298,12 @@
+ 
+ .TP
+ \fBnoolddir\fR
+-Logs are rotated in the same directory the log normally resides in (this 
++Logs are rotated in the directory they normally resides in (this 
+ overrides the \fBolddir\fR option).
+ 
+ .TP
+ \fBnosharedscripts\fR
+-Run \fBprerotate\fR and \fBpostrotate\fR scripts for every script which
++Run \fBprerotate\fR and \fBpostrotate\fR scripts for every log which
+ is rotated (this is the default, and overrides the \fBsharedscripts\fR
+ option).
+ 
+@@ -315,16 +324,15 @@
+ \fBpostrotate\fR/\fBendscript\fR
+ The lines between \fBpostrotate\fR and \fBendscript\fR (both of which
+ must appear on lines by themselves) are executed after the log file is
+-rotated. These directives may only appear inside of a log file definition.
+-See \fBprerotate\fR as well.
++rotated. These directives may only appear inside a log file definition.
++See also \fBprerotate\fR.
+ 
+ .TP
+ \fBprerotate\fR/\fBendscript\fR
+ The lines between \fBprerotate\fR and \fBendscript\fR (both of which
+ must appear on lines by themselves) are executed before the log file is
+ rotated and only if the log will actually be rotated. These directives
+-may only appear inside of a log file definition.  See \fBpostrotate\fR
+-as well.
++may only appear inside a log file definition.  See also \fBpostrotate\fR.
+ 
+ .TP
+ \fBfirstaction\fR/\fBendscript\fR
+@@ -340,30 +348,31 @@
+ must appear on lines by themselves) are executed once after all log
+ files that match the wildcarded pattern are rotated, after postrotate script
+ is run and only if at least one log is rotated. These directives may only
+-appear inside of a log file definition. See \fBlastaction\fR as well.
++appear inside a log file definition. See also \fBlastaction\fR.
+ 
+ .TP
+ \fBrotate \fIcount\fR
+-Log files are rotated <count> times before being removed or mailed to the
++Log files are rotated \fIcount\fR times before being removed or mailed to the
+ address specified in a \fBmail\fR directive. If \fIcount\fR is 0, old versions
+ are removed rather then rotated.
+ 
+ .TP
+-\fBsize \fIsize\fR
++\fBsize \fIsize\fR[\fBG\fR|\fBM\fR|\fBk\fR]
+ Log files are rotated when they grow bigger then \fIsize\fR bytes. If
+ \fIsize\fR is followed by \fIM\fR, the size if assumed to be in megabytes.
+-If the \fIk\fR is used, the size is in kilobytes. So \fBsize 100\fR,
+-\fIsize 100k\fR, and \fIsize 100M\fR are all valid.
++If the \fIG\fR suffix is used, the size is in gigabytes.
++If the \fIk\fR suffix is used, the size is in kilobytes. So \fBsize 100\fR,
++\fIsize 100k\fR, \fIsize 100M\fR and \fIsize 1G\fR are all valid.
+ 
+ .TP
+ \fBsharedscripts\fR
+-Normally, \fBprescript\fR and \fBpostscript\fR scripts are run for each
++Normally, \fBprerotate\fR and \fBpostrotate\fR scripts are run for each
+ log which is rotated, meaning that a single script may be run multiple
+ times for log file entries which match multiple files (such as the 
+-/var/log/news/* example). If \fBsharedscript\fR is specified, the scripts
++\fI/var/log/news/*\fR example). If \fBsharedscript\fR is specified, the scripts
+ are only run once, no matter how many logs match the wildcarded pattern.
+ However, if none of the logs in the pattern require rotating, the scripts
+-will not be run at all. This option overrides the \fbnosharedscripts\fR
++will not be run at all. This option overrides the \fBnosharedscripts\fR
+ option.
+ 
+ .TP
+@@ -377,18 +386,21 @@
+ .TP
+ \fBtabooext\fR [+] \fIlist\fR
+ The current taboo extension list is changed (see the \fBinclude\fR directive
+-for information on the taboo extensions). If a + precedes the list of
+-extensions, the current taboo extension list is augmented, otherwise it
++for information on the taboo extensions). If a \fB+\fR precedes \fIlist\fR,
++the current taboo extension list is augmented by \fIlist\fR, otherwise it
+ is replaced. At startup, the taboo extension list 
+-contains .rpmorig, .rpmsave, ,v, .swp, .rpmnew, and ~.
++contains .rpmorig, .rpmsave, .dpkg-dist, .dpkg-old, .dpkg-new, .disabled,
++,v, .swp, .rpmnew, and ~. The members of the list are separated by spaces,
++not commas.
++
+ 
+ .TP
+ \fBweekly\fR
+ Log files are rotated if the current weekday is less then the weekday
+ of the last rotation or if more then a week has passed since the last
+ rotation. This is normally the same as rotating logs on the first day
+-of the week, but it works better if \fIlogrotate\fR is not run every
+-night.
++of the week, but if \fBlogrotate\fR is not being run every night a log 
++rotation will happen at the first valid opportunity.
+ 
+ .SH FILES
+ .PD 0
+@@ -400,11 +412,16 @@
+ Configuration options.
+ 
+ .SH SEE ALSO
+-.IR gzip (1)
++.BR gzip (1)
++
++.SH NOTES
++The \fBkillall\fR(1) program in Debian is found in the \fIpsmisc\fR package.
+ 
+ .SH AUTHORS
+ .nf
+ Erik Troan <ewt@redhat.com>
+ .nf
+ Preston Brown <pbrown@redhat.com>
++.nf
++Corrections and changes for Debian by Paul Martin <pm@debian.org>
+ .fi

=== added file 'debian/patches/51-man-189243.dpatch'
--- debian/patches/51-man-189243.dpatch	1970-01-01 00:00:00 +0000
+++ debian/patches/51-man-189243.dpatch	2004-06-11 13:51:34 +0000
@@ -0,0 +1,46 @@
+#!/bin/sh -e
+## 51-man-189243.dpatch by Paul Martin <pm@debian.org>
+##
+## All lines beginning with `## DP:' are a description of the patch.
+## DP: Fixes bug #189243
+
+if [ $# -lt 1 ]; then
+    echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+    exit 1
+fi
+
+[ -f debian/patches/00patch-opts ] && . debian/patches/00patch-opts
+patch_opts="${patch_opts:--f --no-backup-if-mismatch}"
+
+case "$1" in
+       -patch) patch $patch_opts -p1 < $0;;
+       -unpatch) patch $patch_opts -p1 -R < $0;;
+        *)
+                echo >&2 "`basename $0`: script expects -patch|-unpatch as argument"
+                exit 1;;
+esac
+
+exit 0
+
+@DPATCH@
+diff -urNad /home/pm/deb/logrotate/logrotate-3.7/logrotate.8 logrotate-3.7/logrotate.8
+--- /home/pm/deb/logrotate/logrotate-3.7/logrotate.8	2004-02-11 21:18:23.000000000 +0000
++++ logrotate-3.7/logrotate.8	2004-02-11 21:18:54.000000000 +0000
+@@ -83,7 +83,7 @@
+     rotate 5
+     weekly
+     postrotate
+-        /sbin/killall -HUP syslogd
++        /usr/bin/killall -HUP syslogd
+     endscript
+ }
+ 
+@@ -93,7 +93,7 @@
+     size=100k
+     sharedscripts
+     postrotate
+-        /sbin/killall -HUP httpd
++        /usr/bin/killall -HUP httpd
+     endscript
+ }
+ 

=== added file 'debian/patches/60-compressowner.dpatch'
--- debian/patches/60-compressowner.dpatch	1970-01-01 00:00:00 +0000
+++ debian/patches/60-compressowner.dpatch	2004-06-11 13:51:34 +0000
@@ -0,0 +1,60 @@
+#! /bin/sh -e
+## 60-compressowner.dpatch by Paul Martin <pm@debian.org>
+##
+## All lines beginning with `## DP:' are a description of the patch.
+## DP: No description.
+
+if [ $# -lt 1 ]; then
+    echo "`basename $0`: script expects -patch|-unpatch as argument" >&2
+    exit 1
+fi
+
+[ -f debian/patches/00patch-opts ] && . debian/patches/00patch-opts
+patch_opts="${patch_opts:--f --no-backup-if-mismatch} ${2:+-d $2}"
+
+case "$1" in
+    -patch) patch -p1 ${patch_opts} < $0;;
+    -unpatch) patch -R -p1 ${patch_opts} < $0;;
+    *)
+        echo "`basename $0`: script expects -patch|-unpatch as argument" >&2
+        exit 1;;
+esac
+
+exit 0
+
+@DPATCH@
+diff -urNad /home/pm/deb/logrotate/logrotate-3.7/logrotate.c logrotate-3.7/logrotate.c
+--- /home/pm/deb/logrotate/logrotate-3.7/logrotate.c	2004-05-10 19:30:36.000000000 +0100
++++ logrotate-3.7/logrotate.c	2004-05-10 19:31:04.000000000 +0100
+@@ -128,6 +128,7 @@
+     int outFile;
+     int i;
+     int status;
++    struct stat sb;
+ 
+     fullCommand = alloca(sizeof(*fullCommand) * 
+ 			 (log->compress_options_count + 2));
+@@ -145,12 +146,22 @@
+ 	return 1;
+     }
+     
+-    if ((outFile = open(compressedName, O_RDWR | O_CREAT | O_TRUNC, 0666)) < 0) {
++    fstat(inFile, &sb);
++
++    if ((outFile = open(compressedName, O_RDWR | O_CREAT | O_TRUNC, sb.st_mode)) < 0) {
+ 	message(MESS_ERROR, "unable to open %s for compressed output\n", 
+ 		compressedName);
+ 	close(inFile);
+ 	return 1;
+     }
++    
++    if (fchown(outFile, sb.st_uid, sb.st_gid)) {
++	message(MESS_ERROR, "unable to change owner of output file %s\n",
++	        compressedName);
++	close(inFile);
++	close(outFile);
++	return 1;
++    }
+ 
+     message(MESS_DEBUG, "compressing log with: %s\n", fullCommand[0]);
+ 

=== modified file 'debian/rules'
--- debian/rules	2002-04-23 23:16:43 +0000
+++ debian/rules	2004-06-11 13:51:34 +0000
@@ -5,36 +5,52 @@
 # Uncomment this to turn on verbose mode.
 #export DH_VERBOSE=1
 
-# This is the debhelper compatability version to use.
-export DH_COMPAT=3
+CFLAGS = -Wall -g
+INSTALL = install
+INSTALL_FILE    = $(INSTALL) -p    -o root -g root  -m  644
+INSTALL_PROGRAM = $(INSTALL) -p    -o root -g root  -m  755
+INSTALL_SCRIPT  = $(INSTALL) -p    -o root -g root  -m  755
+INSTALL_DIR     = $(INSTALL) -p -d -o root -g root  -m  755
+
+ifneq (,$(findstring noopt,$(DEB_BUILD_OPTIONS)))
+CFLAGS += -O0
+else
+CFLAGS += -O2
+endif
+ifeq (,$(findstring nostrip,$(DEB_BUILD_OPTIONS)))
+INSTALL_PROGRAM += -s
+endif
+
 
 configure: configure-stamp
-configure-stamp:
+configure-stamp: patch
 	dh_testdir
 	# Add here commands to configure the package.
 
 	touch configure-stamp
 
 build: configure-stamp build-stamp
-build-stamp:
+build-stamp: patch
 	dh_testdir
 
-	# Add here commands to compile the package.
-	RPM_OPT_FLAGS=-O2 $(MAKE)
+	#$(MAKE) RPM_OPT_FLAGS="$(CFLAGS)" VERSION="3.7" WITH_SELINUX="yes"
+	#mv $(CURDIR)/logrotate $(CURDIR)/logrotate.selinux
+	#rm $(CURDIR)/logrotate.o
+
+	$(MAKE) RPM_OPT_FLAGS="$(CFLAGS)" VERSION="3.7"
 
 	touch build-stamp
 
-clean:
+clean: clean-patched unpatch
+clean-patched:
 	dh_testdir
 	dh_testroot
 	rm -f build-stamp configure-stamp
 
-	# Add here commands to clean up after the build process.
 	-$(MAKE) clean
 	rm -f .depend
 
-	dh_clean
-	rm -f debian/cron.daily
+	dh_clean logrotate.selinux
 
 install: build
 	dh_testdir
@@ -43,11 +59,21 @@
 	dh_installdirs
 
 	# Add here commands to install the package into debian/logrotate.
-	$(MAKE) install PREFIX=$(CURDIR)/debian/logrotate
-	install -m 644 $(CURDIR)/examples/logrotate.cron \
-	  $(CURDIR)/debian/cron.daily
-	install -m 644 $(CURDIR)/debian/logrotate.conf \
+	$(INSTALL_PROGRAM) $(CURDIR)/logrotate \
+	  $(CURDIR)/debian/logrotate/usr/sbin/
+	#$(INSTALL_PROGRAM) $(CURDIR)/logrotate.selinux \
+	#  $(CURDIR)/debian/logrotate-selinux/usr/sbin/logrotate
+
+	$(INSTALL_FILE) $(CURDIR)/debian/logrotate.conf \
 	  $(CURDIR)/debian/logrotate/etc/logrotate.conf
+	#$(INSTALL_FILE) $(CURDIR)/debian/logrotate.conf \
+	#  $(CURDIR)/debian/logrotate-selinux/etc/logrotate.conf
+
+	# reportbug script
+	$(INSTALL_SCRIPT) $(CURDIR)/debian/logrotate.bug \
+	  $(CURDIR)/debian/logrotate/usr/share/bug/logrotate/script
+	#$(INSTALL_SCRIPT) $(CURDIR)/debian/logrotate.bug \
+	#  $(CURDIR)/debian/logrotate-selinux/usr/share/bug/logrotate/script
 
 
 
@@ -69,7 +95,7 @@
 #	dh_installmime
 #	dh_installinit
 	dh_installcron
-	dh_installman
+	dh_installman logrotate.8
 #	dh_installinfo
 #	dh_undocumented
 	dh_installchangelogs CHANGES
@@ -86,4 +112,16 @@
 	dh_builddeb
 
 binary: binary-indep binary-arch
-.PHONY: build clean binary-indep binary-arch binary install configure
+
+patch: patch-stamp
+patch-stamp:
+	dpatch apply-all
+	touch patch-stamp
+	# dpatch call-all -a=pkg-info >patch-stamp
+
+unpatch:
+	dpatch deapply-all
+	rm -rf patch-stamp debian/patched
+
+.PHONY: build clean binary-indep binary-arch binary install configure \
+ patch unpatch clean-patched

=== modified file 'examples/logrotate-default'
--- examples/logrotate-default	2002-04-23 23:16:43 +0000
+++ examples/logrotate-default	2004-06-11 13:51:34 +0000
@@ -14,7 +14,7 @@
 # RPM packages drop log rotation information into this directory
 include /etc/logrotate.d
 
-# no packages own lastlog or wtmp -- we'll rotate them here
+# no packages own wtmp -- we'll rotate them here
 /var/log/wtmp {
     monthly
     create 0664 root utmp

=== modified file 'examples/logrotate.cron'
--- examples/logrotate.cron	2002-04-23 23:16:43 +0000
+++ examples/logrotate.cron	2004-06-11 13:51:34 +0000
@@ -1,3 +1,8 @@
 #!/bin/sh
 
 /usr/sbin/logrotate /etc/logrotate.conf
+EXITVALUE=$?
+if [ $EXITVALUE != 0 ]; then
+    /usr/bin/logger -t logrotate "ALERT exited abnormally with [$EXITVALUE]"
+fi
+exit 0

=== modified file 'logrotate.8'
--- logrotate.8	2002-04-23 23:16:43 +0000
+++ logrotate.8	2004-06-11 13:51:34 +0000
@@ -1,10 +1,9 @@
-.TH LOGROTATE 8 "Thu Aug 02 2001"
+.TH LOGROTATE 8 "Wed Nov 5 2002" "Red Hat Linux" "System Administrator's Manual"
 .UC 4
 .SH NAME
 logrotate \- rotates, compresses, and mails system logs
 .SH SYNOPSIS
-\fBlogrotate\fR [\fB\-dv\fR] [\fB\-f\fR|\fB\-\-force\fR]
-[\fB\-s\fR|\fB-\-state\ \fIstatefile\fR] \fIconfig_file\fR ..
+\fBlogrotate\fR [-dv] [-f|--force] [-s|--state \fIfile\fR] \fIconfig_file\fR+
 .SH DESCRIPTION
 \fBlogrotate\fR is designed to ease administration of systems that generate
 large numbers of log files.  It allows automatic rotation, compression, 
@@ -12,16 +11,16 @@
 weekly, monthly, or when it grows too large.
 .P
 Normally, \fBlogrotate\fR is run as a daily cron job.  It will not modify
-a log more than once in one day unless the criterion for that log is
-based on the log's size and \fBlogrotate\fR is being run more than once
-each day, or unless the \fB-f\fR or \fB-\-force\fR option is used. 
+a log multiple times in one day unless the criterium for that log is
+based on the log's size and \fBlogrotate\fR is being run multiple times
+each day, or unless the \fB-f\fR or \fB-force\fR option is used. 
 .P
 Any number of config files may be given on the command line. Later config
 files may override the options given in earlier files, so the order
-in which the \fBlogrotate\fR config files are listed is important.
+in which the \fBlogrotate\fR config files are listed in is important.
 Normally, a single config file which includes any other config files
 which are needed should be used.  See below for more information on how
-to use the \fBinclude\fR directive to accomplish this.  If a directory
+to use the \fIinclude\fR directive to accomplish this.  If a directory
 is given on the command line, every file in that directory is used as
 a config file.
 .P
@@ -37,35 +36,37 @@
 be made to the logs or to the \fBlogrotate\fR state file.
 
 .TP
-\fB-f\fR, \fB-\-force\fR
+\fB-f, -\-force\fR
 Tells \fBlogrotate\fR to force the rotation, even if it doesn't think
 this is necessary.  Sometimes this is useful after adding new entries to
-a \fBlogrotate\fR config file, or if old log files have been removed
-by hand, as the new files will be created, and logging will continue
-correctly.
-
-.TP
-\fB-s\fR, \fB-\-state\fR \fIstatefile\fR
+\fBlogrotate\fR, or if old log files have been removed by hand, as the
+new files will be created, and logging will continue correctly.
+
+.TP
+\fB-m, -\-mail <command>\fR
+Tells \fBlogrotate\fR which command to use when mailing logs. This
+command should accept two arguments: 1) the subject of the message, and
+2) the recipient. The command must then read a message on standard input
+and mail it to the recipient. The default mail command is \fB/bin/mail
+-s\fR.
+
+.TP
+\fB-s, -\-state <statefile>\fR
 Tells \fBlogrotate\fR to use an alternate state file.  This is useful
-if \fBlogrotate\fR is being run as different users for various sets of
+if logrotate is being run as a different user for various sets of
 log files.  The default state file is \fI/var/lib/logrotate/status\fR.
 
 .TP
 \fB-\-usage\fR
 Prints a short usage message.
 
-.TP
-\fB-v\fR, \fB--verbose\fR
-Display messages during rotation.
-
-
 .SH CONFIGURATION FILE
 
 \fBlogrotate\fR reads everything about the log files it should be handling
 from the series of configuration files specified on the command line.  Each
 configuration file can set global options (local definitions override
 global ones, and later definitions override earlier ones) and specify
-some logfiles to rotate. A simple configuration file looks like this:
+a logfile to rotate. A simple configuration file looks like this:
 
 .nf
 .ta +3i
@@ -76,7 +77,7 @@
     rotate 5
     weekly
     postrotate
-        /sbin/killall -HUP syslogd
+	/sbin/killall -HUP syslogd
     endscript
 }
 
@@ -86,16 +87,17 @@
     size=100k
     sharedscripts
     postrotate
-        /sbin/killall -HUP httpd
+	/sbin/killall -HUP httpd
     endscript
 }
 
-/var/log/news/* {
+/var/log/news/news.crit {
     monthly
     rotate 2
+    olddir /var/log/news/old
     missingok
     postrotate
-        kill -HUP `cat /var/run/inn.pid`
+	kill -HUP `cat /var/run/inn.pid`
     endscript
     nocompress
 }
@@ -105,9 +107,9 @@
 The first few lines set global options; in the example, logs are
 compressed after they are rotated.  Note that comments may appear
 anywhere in the config file as long as the first non-whitespace
-character on the line is a \fB#\fR.
+character on the line is a #.
 
-The next section of the config file defines how to handle the log file
+The next section of the config files defined how to handle the log file
 \fI/var/log/messages\fR. The log will go through five weekly rotations before
 being removed. After the log file has been rotated (but before the old
 version of the log has been compressed), the command 
@@ -115,13 +117,13 @@
 
 The next section defines the parameters for both
 \fI/var/log/httpd/access.log\fR and \fI/var/log/httpd/error.log\fR.
-Each is rotated whenever it grows over 100k in size, and the old logs
+They are rotated whenever is grows over 100k is size, and the old logs
 files are mailed (uncompressed) to www@my.org after going through 5
 rotations, rather then being removed. The \fBsharedscripts\fR means that
 the \fBpostrotate\fR script will only be run once, not once for each
-log which is rotated. Note that log file names may be enclosed in
-quotes (and that quotes are required if the name contains spaces).
-Normal shell quoting rules apply, with \fB'\fR, \fB"\fR, and \fB\\\fR
+log which is rotated. Note that the double quotes around the first filename
+at the beginning of this section allows logrotate to rotate logs with
+spaces in the name. Normal shell quoting rules apply, with ', ", and \\
 characters supported.
 
 The last section defines the parameters for all of the files in
@@ -129,27 +131,27 @@
 considered a single rotation directive and if errors occur for more then
 one file, the log files are not compressed.
 
-If the directory \fI/var/log/news\fR does not exist, this will cause 
-\fBlogrotate\fR to report an error. This error cannot be stopped with
-the \fBmissingok\fR directive.
+Please use wildcards with caution.  If you specify *, \fBlogrotate\fR will
+rotate all files, including previously rotated ones.  A way around this
+is to use the \fBolddir\fR directive or a more exact wildcard (such as *.log).
 
 Here is more information on the directives which may be included in
 a \fBlogrotate\fR configuration file:
 
 .TP
 \fBcompress\fR
-Old versions of log files are compressed with \fBgzip\fR(1) by default.
-See also \fBnocompress\fR. 
+Old versions of log files are compressed with \fBgzip\fR by default. See also
+\fBnocompress\fR. 
 
 .TP
 \fBcompresscmd\fR
 Specifies which command to use to compress log files.  The default is
-\fBgzip\fR(1).  See also \fBcompress\fR.
+\fBgzip\fR.  See also \fBcompress\fR.
 
 .TP
 \fBuncompresscmd\fR
 Specifies which command to use to uncompress log files.  The default is
-\fBgunzip\fR(1).
+\fBgunzip\fR.
 
 .TP
 \fBcompressext\fR
@@ -163,10 +165,18 @@
 in use.  The default, for \fBgzip\fR, is "-9" (maximum compression).
 
 .TP
+\fBcopy\fR
+Make a copy of the log file, but don't change the original at all.
+This option can be used, for instance, to make a snapshot of the current
+log file, or when some other utility needs to truncate or pare the file.
+When this option is used, the \fBcreate\fR option will have no effect,
+as the old log file stays in place.
+
+.TP
 \fBcopytruncate\fR
-Truncate the original log file to zero size in place after creating a copy,
-instead of moving the old log file and optionally creating a new one.
-It can be used when some program cannot be told to close its logfile
+Truncate the original log file in place after creating a copy,
+instead of moving the old log file and optionally creating a new one,
+It can be used when some program can not be told to close its logfile
 and thus might continue writing (appending) to the previous log file forever.
 Note that there is a very small time slice between copying the file and
 truncating it, so some logging data might be lost.
@@ -178,7 +188,7 @@
 Immediately after rotation (before the \fBpostrotate\fR script is run)
 the log file is created (with the same name as the log file just rotated).
 \fImode\fR specifies the mode for the log file in octal (the same
-as \fBchmod\fR(2)), \fIowner\fR specifies the user name who will own the
+as \fBchmod(2)\fR), \fIowner\fR specifies the user name who will own the
 log file, and \fIgroup\fR specifies the group the log file will belong
 to. Any of the log file attributes may be omitted, in which case those
 attributes for the new file will use the same values as the original log
@@ -192,35 +202,36 @@
 .TP
 \fBdelaycompress\fR
 Postpone compression of the previous log file to the next rotation cycle.
-This only has effect when used in combination with \fBcompress\fR.
-It can be used when some program cannot be told to close its logfile
+This has only effect when used in combination with \fBcompress\fR.
+It can be used when some program can not be told to close its logfile
 and thus might continue writing to the previous log file for some time.
 
 .TP
 \fBextension \fIext\fR
 Log files are given the final extension \fIext\fR after rotation. If 
-compression is used, the compression extension (normally \fI.gz\fR)
+compression is used, the compression extension (normally \fB.gz\fR)
 appears after \fIext\fR.
 
 .TP
 \fBifempty\fR
-Rotate the log file even if it is empty, overriding the \fBnotifempty\fR
-option (\fBifempty\fR is the default).
+Rotate the log file even if it is empty, overiding the \fBnotifempty\fR
+option (ifempty is the default).
 
 .TP
 \fBinclude \fIfile_or_directory\fR
-Reads the file given as an argument as if it was included inline where
-the \fBinclude\fR directive appears. If a directory is given, most of the
-files in that directory are read before processing of the including file
-continues. The only files which are ignored are files which are not regular
-files (such as directories and named pipes) and files whose names end
-with one of the taboo extensions, as specified by the \fBtabooext\fR
-directive.  The \fBinclude\fR directive may not appear inside a log
-file definition.
+Reads the file given as an argument as if it was included inline
+where the \fBinclude\fR directive appears. If a directory is given,
+most of the files in that directory are read in alphabetic order
+before processing of the including file continues. The only files
+which are ignored are files which are not regular files (such as
+directories and named pipes) and files whose names end with one of
+the taboo extensions, as specified by the \fBtabooext\fR directive.
+The \fBinclude\fR directive may not appear inside of a log file
+definition.
 
 .TP
 \fBmail \fIaddress\fR
-When a log is rotated out of existence, it is mailed to \fIaddress\fR. If
+When a log is rotated out-of-existence, it is mailed to \fIaddress\fR. If
 no mail should be generated by a particular log, the \fBnomail\fR directive
 may be used.
 
@@ -246,7 +257,13 @@
 
 .TP
 \fBnocompress\fR
-Old versions of log files are not compressed. See also \fBcompress\fR. 
+Old versions of log files are not compressed with \fBgzip\fR. See also
+\fBcompress\fR. 
+
+.TP
+\fBnocopy\fR
+Do not copy the original log file and leave it in place.
+(this overrides the \fBcopy\fR option).
 
 .TP
 \fBnocopytruncate\fR
@@ -264,7 +281,7 @@
 
 .TP
 \fBnomail\fR
-Do not mail old log files to any address.
+Don't mail old log files to any address.
 
 .TP
 \fBnomissingok\fR
@@ -272,12 +289,12 @@
 
 .TP
 \fBnoolddir\fR
-Logs are rotated in the directory they normally reside in (this 
+Logs are rotated in the same directory the log normally resides in (this 
 overrides the \fBolddir\fR option).
 
 .TP
 \fBnosharedscripts\fR
-Run \fBprerotate\fR and \fBpostrotate\fR scripts for every log that
+Run \fBprerotate\fR and \fBpostrotate\fR scripts for every script which
 is rotated (this is the default, and overrides the \fBsharedscripts\fR
 option).
 
@@ -287,66 +304,90 @@
 
 .TP
 \fBolddir \fIdirectory\fR
-Logs are moved into \fIdirectory\fR for rotation. The \fIdirectory\fR must
-be on the same physical device as the log file being rotated. When this
-option is used all old versions of the log end up in \fIdirectory\fR.  This
-option may be overridden by the \fBnoolddir\fR option.
+Logs are moved into \fIdirectory\fR for rotation. The \fIdirectory\fR
+must be on the same physical device as the log file being rotated,
+and is assumed to be relative to the directory holding the log file
+unless an absolute path name is specified. When this option is used all
+old versions of the log end up in \fIdirectory\fR.  This option may be
+overriden by the \fBnoolddir\fR option.
 
 .TP
 \fBpostrotate\fR/\fBendscript\fR
 The lines between \fBpostrotate\fR and \fBendscript\fR (both of which
 must appear on lines by themselves) are executed after the log file is
-rotated. These directives may only appear inside a log file definition.
-See also \fBprerotate\fR.
+rotated. These directives may only appear inside of a log file definition.
+See \fBprerotate\fR as well.
 
 .TP
 \fBprerotate\fR/\fBendscript\fR
 The lines between \fBprerotate\fR and \fBendscript\fR (both of which
 must appear on lines by themselves) are executed before the log file is
-rotated. These directives may only appear inside a log file definition.
-See also \fBpostrotate\fR.
+rotated and only if the log will actually be rotated. These directives
+may only appear inside of a log file definition.  See \fBpostrotate\fR
+as well.
+
+.TP
+\fBfirstaction\fR/\fBendscript\fR
+The lines between \fBfirstaction\fR and \fBendscript\fR (both of which
+must appear on lines by themselves) are executed once before all log
+files that match the wildcarded pattern are rotated, before prerotate script
+is run and only if at least one log will actually be rotated. These directives
+may only appear inside of a log file definition. See \fBlastaction\fR as well.
+
+.TP
+\fBlastaction\fR/\fBendscript\fR
+The lines between \fBlastaction\fR and \fBendscript\fR (both of which
+must appear on lines by themselves) are executed once after all log
+files that match the wildcarded pattern are rotated, after postrotate script
+is run and only if at least one log is rotated. These directives may only
+appear inside of a log file definition. See \fBlastaction\fR as well.
 
 .TP
 \fBrotate \fIcount\fR
-Log files are rotated \fIcount\fR times before being removed or mailed to the
+Log files are rotated <count> times before being removed or mailed to the
 address specified in a \fBmail\fR directive. If \fIcount\fR is 0, old versions
 are removed rather then rotated.
 
 .TP
+\fBsize \fIsize\fR
+Log files are rotated when they grow bigger then \fIsize\fR bytes. If
+\fIsize\fR is followed by \fIM\fR, the size if assumed to be in megabytes.
+If the \fIk\fR is used, the size is in kilobytes. So \fBsize 100\fR,
+\fIsize 100k\fR, and \fIsize 100M\fR are all valid.
+
+.TP
 \fBsharedscripts\fR
-Normally, \fBprerotate\fR and \fBpostrotate\fR scripts are run for each
+Normally, \fBprescript\fR and \fBpostscript\fR scripts are run for each
 log which is rotated, meaning that a single script may be run multiple
 times for log file entries which match multiple files (such as the 
-\fI/var/log/news/*\fR example). If \fBsharedscript\fR is specified, the scripts
+/var/log/news/* example). If \fBsharedscript\fR is specified, the scripts
 are only run once, no matter how many logs match the wildcarded pattern.
-A side effect of this option is that the scripts are always executed, even
-if no logs are rotated. If this directive is not specified, the scripts
-are run only if logs are actually rotated. This overrides the
-\fBnosharedscripts\fR option.
-
-.TP
-\fBsize \fIsize\fR[\fBM\fR|\fBk\fR]
-Log files are rotated when they grow bigger than \fIsize\fR bytes. If
-\fIsize\fR is followed by \fBM\fR, the size is assumed to be in megabytes.
-If \fBk\fR is used, the size is in kilobytes. So \fBsize 100\fR,
-\fBsize 100k\fR, and \fBsize 100M\fR are all valid.
-
-.TP
-\fBtabooext\fR [\fB+\fR] \fIlist\fR
+However, if none of the logs in the pattern require rotating, the scripts
+will not be run at all. This option overrides the \fbnosharedscripts\fR
+option.
+
+.TP
+\fBstart \fIcount\fR
+This is the number to use as the base for rotation. For example, if
+you specify 0, the logs will be created with a .0 extension as they are
+rotated from the original log files.  If you specify 9, log files will
+be created with a .9, skipping 0-8.  Files will still be rotated the
+number of times specified with the \fBcount\fR directive.
+
+.TP
+\fBtabooext\fR [+] \fIlist\fR
 The current taboo extension list is changed (see the \fBinclude\fR directive
-for information on the taboo extensions). If \fB+\fR precedes \fIlist\fR,
-the current taboo extension list is augmented by \fIlist\fR, otherwise 
-it is replaced. At startup, the taboo extension list 
-contains .rpmorig, .rpmsave, .dpkg-dist, .dpkg-old, .dpkg-new, .disabled,
-,v, .swp, .rpmnew, and ~. The members of the list are separated by spaces,
-not commas.
+for information on the taboo extensions). If a + precedes the list of
+extensions, the current taboo extension list is augmented, otherwise it
+is replaced. At startup, the taboo extension list 
+contains .rpmorig, .rpmsave, ,v, .swp, .rpmnew, and ~.
 
 .TP
 \fBweekly\fR
-Log files are rotated if the current weekday is less than the weekday
-of the last rotation or if more than a week has passed since the last
+Log files are rotated if the current weekday is less then the weekday
+of the last rotation or if more then a week has passed since the last
 rotation. This is normally the same as rotating logs on the first day
-of the week, but works better if \fBlogrotate\fR is not run every
+of the week, but it works better if \fIlogrotate\fR is not run every
 night.
 
 .SH FILES
@@ -359,10 +400,7 @@
 Configuration options.
 
 .SH SEE ALSO
-.BR gzip (1)
-
-.SH NOTES
-The \fBkillall\fR(1) program in Debian is found in the \fIpsmisc\fR package.
+.IR gzip (1)
 
 .SH AUTHORS
 .nf

=== modified file 'logrotate.c'
--- logrotate.c	2002-04-23 23:16:43 +0000
+++ logrotate.c	2004-06-11 13:51:34 +0000
@@ -8,9 +8,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <time.h>
 #include <unistd.h>
 
+#ifdef WITH_SELINUX
+#include <selinux/selinux.h>
+static security_context_t prev_context=NULL;
+int selinux_enabled=0;
+#endif
+
 #include "basenames.h"
 #include "log.h"
 #include "logrotate.h"
@@ -18,21 +25,27 @@
 typedef struct {
     char * fn;
     struct tm lastRotated;	/* only tm.mon, tm_mday, tm_year are good! */
+    struct stat sb;
+    int doRotate;
 } logState;
 
+struct stateSet {
+    logState * states;
+    int numStates;
+};
+
 #define NO_MODE ((mode_t) -1)
 #define NO_UID  ((uid_t) -1)
 #define NO_GID  ((gid_t) -1)
 
 int debug = 0;
 char * mailCommand = DEFAULT_MAIL_COMMAND;
+time_t nowSecs = 0;
 
-static logState * findState(const char * fn, logState ** statesPtr, 
-			    int * numStatesPtr) {
+static logState * findState(const char * fn, struct stateSet * sip) {
     int i;
-    logState * states = *statesPtr;
-    int numStates = *numStatesPtr;
-    time_t nowSecs = time(NULL);
+    logState * states = sip->states;
+    int numStates = sip->numStates;
     struct tm now = *localtime(&nowSecs);
     time_t lr_time;
 
@@ -44,6 +57,7 @@
 	states = realloc(states, sizeof(*states) * numStates);
 	states[i].fn = strdup(fn);
 	memset(&states[i].lastRotated, 0, sizeof(states[i].lastRotated));
+	states[i].doRotate = 0;
 
 	states[i].lastRotated.tm_mon = now.tm_mon;
 	states[i].lastRotated.tm_mday = now.tm_mday;
@@ -53,8 +67,8 @@
 	lr_time = mktime(&states[i].lastRotated);
 	states[i].lastRotated = *localtime(&lr_time);
 
-	*statesPtr = states;
-	*numStatesPtr = numStates;
+	sip->states = states;
+	sip->numStates = numStates;
     }
 
     return (states + i);
@@ -62,9 +76,9 @@
 
 static int runScript(char * logfn, char * script) {
     int fd;
-    char filespec[32];
-    char * cmd;
+    char *filespec;
     int rc;
+    char buf[256];
 
     if (debug) {
 	message(MESS_DEBUG, "running script with arg %s: \"%s\"\n", 
@@ -72,20 +86,22 @@
 	return 0;
     }
 
-    strcpy(filespec, "/tmp/logrotate.XXXXXX");
-    if ((fd = mkstemp(filespec)) < 0 || fchmod(fd, 0700)) {
+    filespec = buf;
+    snprintf(buf, sizeof(buf), "%s/logrotate.XXXXXX", getenv("TMPDIR") ?: "/tmp");
+    fd = -1;
+    if (!filespec || (fd = mkstemp(filespec)) < 0 || fchmod(fd, 0700)) {
 	message(MESS_DEBUG, "error creating %s: %s\n", filespec,
 		strerror(errno));
-	if (fd >= 0) close(fd);
+	if (fd >= 0) {
+	    close(fd);
+	    unlink(filespec);
+	}
 	return -1;
     }
 
-    if (write(fd, "#!/bin/sh\n\n", 11) != 11) {
-	close(fd);
-	unlink(filespec);
-	return -1;
-    }
-    if (write(fd, script, strlen(script)) != strlen(script)) {
+    if (write(fd, "#!/bin/sh\n\n", 11) != 11 ||
+	write(fd, script, strlen(script)) != strlen(script)) {
+	message(MESS_DEBUG, "error writing %s\n", filespec);
 	close(fd);
 	unlink(filespec);
 	return -1;
@@ -93,16 +109,142 @@
 
     close(fd);
 
-    cmd = alloca(strlen(filespec) + strlen(logfn) + 20);
-    sprintf(cmd, "/bin/sh %s '%s'", filespec, logfn);
-    rc = system(cmd);
+    if (!fork()) {
+	execlp(filespec, logfn, NULL);
+	exit(1);
+    }
+
+    wait(&rc);
 
     unlink(filespec);
 
     return rc;
 }
 
-static int copyTruncate(char * currLog, char * saveLog, struct stat * sb) {
+static int compressLogFile(char * name, logInfo * log) {
+    char * compressedName;
+    const char ** fullCommand;
+    int inFile;
+    int outFile;
+    int i;
+    int status;
+
+    fullCommand = alloca(sizeof(*fullCommand) * 
+			 (log->compress_options_count + 2));
+    fullCommand[0] = log->compress_prog;
+    for (i = 0; i < log->compress_options_count; i++)
+	fullCommand[i + 1] = log->compress_options_list[i];
+    fullCommand[log->compress_options_count + 1] = NULL;
+    
+    compressedName = alloca(strlen(name) + 
+			    strlen(log->compress_ext) + 2);
+    sprintf(compressedName, "%s%s", name, log->compress_ext);
+
+    if ((inFile = open(name, O_RDONLY)) < 0) {
+	message(MESS_ERROR, "unable to open %s for compression\n", name);
+	return 1;
+    }
+    
+    if ((outFile = open(compressedName, O_RDWR | O_CREAT | O_TRUNC, 0666)) < 0) {
+	message(MESS_ERROR, "unable to open %s for compressed output\n", 
+		compressedName);
+	close(inFile);
+	return 1;
+    }
+
+    message(MESS_DEBUG, "compressing log with: %s\n", fullCommand[0]);
+
+    if (!fork()) {
+	dup2(inFile, 0);
+	close(inFile);
+	dup2(outFile, 1);
+	close(outFile);
+
+	execvp(fullCommand[0], (void *) fullCommand);
+	exit(1);
+    }
+
+    close(inFile);
+    close(outFile);
+
+    wait(&status);
+
+    if (!WIFEXITED(status) || WEXITSTATUS(status)) {
+	message(MESS_ERROR, "failed to compress log %s\n", name);
+	return 1;
+    }
+
+    unlink(name);
+
+    return 0;
+}
+
+static int mailLog(char * logFile, char * mailCommand, char * uncompressCommand, 
+		   char * address, char * subject) {
+    int mailInput;
+    pid_t mailChild, uncompressChild;
+    int mailStatus, uncompressStatus;
+    int uncompressPipe[2];
+    char * mailArgv[] = { mailCommand, "-s", subject, address, NULL };
+    int rc = 0;
+
+    if ((mailInput = open(logFile, O_RDONLY)) < 0) {
+	message(MESS_ERROR, "failed to open %s for mailing: %s\n", logFile,
+		strerror(errno));
+	return 1;
+    }
+
+    if (uncompressCommand) {
+	pipe(uncompressPipe);
+	if (!(uncompressChild = fork())) {
+	    /* uncompress child */
+	    dup2(mailInput, 0);
+	    close(mailInput);
+	    dup2(uncompressPipe[1], 1);
+	    close(uncompressPipe[0]);
+	    close(uncompressPipe[1]);
+
+	    execlp(uncompressCommand, uncompressCommand, NULL);
+	    exit(1);
+	}
+
+	close(mailInput);
+	mailInput = uncompressPipe[0];
+	close(uncompressPipe[1]);
+    }
+
+    if (!(mailChild = fork())) {
+	dup2(mailInput, 0);
+	close(mailInput);
+	close(1);
+
+	execvp(mailArgv[0], mailArgv);
+	exit(1);
+    }
+
+    close(mailInput);
+
+    waitpid(mailChild, &mailStatus, 0);
+
+    if (!WIFEXITED(mailStatus) || WEXITSTATUS(mailStatus)) {
+	message(MESS_ERROR, "mail command failed for %s\n", logFile);
+	rc = 1;
+    }
+
+    if (uncompressCommand) {
+	waitpid(uncompressChild, &uncompressStatus, 0);
+
+	if (!WIFEXITED(uncompressStatus) || WEXITSTATUS(uncompressStatus)) {
+	    message(MESS_ERROR, "uncompress command failed mailing %s\n", 
+		    logFile);
+	    rc = 1;
+	}
+    }
+
+    return rc;
+}
+
+static int copyTruncate(char * currLog, char * saveLog, struct stat * sb, int flags) {
     char buf[BUFSIZ];
     int fdcurr = -1, fdsave = -1;
     ssize_t cnt;
@@ -115,14 +257,48 @@
 		strerror(errno));
 	    return 1;
 	}
-	if ((fdsave = open(saveLog, O_WRONLY | O_CREAT | O_TRUNC,
-		sb->st_mode)) < 0) {
+#ifdef WITH_SELINUX
+	if ((selinux_enabled=(is_selinux_enabled()>0)))
+	  {
+	    security_context_t oldContext;
+	    if (fgetfilecon(fdcurr, &oldContext) >=0) {
+	      if (getfscreatecon(&prev_context) < 0) {
+		message(MESS_ERROR, "error getting default context: %s\n", 
+			strerror(errno));
+		freecon(oldContext);
+		return 1;
+	      }
+	      if (setfscreatecon(oldContext) < 0) {
+		message(MESS_ERROR, "error setting file context %s to %s: %s\n", 
+			saveLog, oldContext,strerror(errno));
+		freecon(oldContext);
+		return 1;
+	      }
+	      freecon(oldContext);
+	    } else {
+	      message(MESS_ERROR, "error getting file context %s: %s\n", currLog,
+		      strerror(errno));
+	      return 1;
+	    }
+	  }
+#endif
+	fdsave = open(saveLog, O_WRONLY | O_CREAT | O_TRUNC,sb->st_mode);
+#ifdef WITH_SELINUX
+	if (selinux_enabled) {
+	  setfscreatecon(prev_context);
+	  if (prev_context!= NULL) {
+	    freecon(prev_context);
+	    prev_context=NULL;
+	  }
+	}
+#endif
+	if (fdsave < 0) {
 	    message(MESS_ERROR, "error creating %s: %s\n", saveLog,
 		strerror(errno));
 	    close(fdcurr);
 	    return 1;
 	}
-	if (fchmod(fdsave, sb->st_mode)) {
+	if (fchmod(fdsave, (S_IRUSR | S_IWUSR) & sb->st_mode)) {
 	    message(MESS_ERROR, "error setting mode of %s: %s\n",
 		saveLog, strerror(errno));
 	    close(fdcurr);
@@ -136,6 +312,13 @@
 	    close(fdsave);
 	    return 1;
 	}
+	if (fchmod(fdsave, sb->st_mode)) {
+	    message(MESS_ERROR, "error setting mode of %s: %s\n",
+		saveLog, strerror(errno));
+	    close(fdcurr);
+	    close(fdsave);
+	    return 1;
+	}
 	while ((cnt = read(fdcurr, buf, sizeof(buf))) > 0) {
 	    if (write(fdsave, buf, cnt) != cnt) {
 		message(MESS_ERROR, "error writing to %s: %s\n", 
@@ -154,28 +337,111 @@
 	}
     }
 
-    message(MESS_DEBUG, "truncating %s\n", currLog);
+    if (flags & LOG_FLAG_COPYTRUNCATE) {
+        message(MESS_DEBUG, "truncating %s\n", currLog);
 
-    if (!debug) {
-	if (ftruncate(fdcurr, 0)) {
-	    message(MESS_ERROR, "error truncating %s: %s\n", currLog,
-		strerror(errno));
-	    close(fdcurr);
-	    close(fdsave);
-	    return 1;
-	} else {
-	    close(fdcurr);
-	    close(fdsave);
-	}
+        if (!debug) {
+	    if (ftruncate(fdcurr, 0)) {
+	        message(MESS_ERROR, "error truncating %s: %s\n", currLog,
+		    strerror(errno));
+	        close(fdcurr);
+	        close(fdsave);
+	        return 1;
+	    } else {
+	        close(fdcurr);
+	        close(fdsave);
+	    }
+        }
+    } else {
+        message(MESS_DEBUG, "Not truncating %s\n", currLog);
+        close(fdcurr);
+        close(fdsave);
     }
 
     return 0;
 }
 
-int rotateSingleLog(logInfo * log, int logNum, logState ** statesPtr, 
-		    int * numStatesPtr, FILE * errorFile) {
+int findNeedRotating(logInfo * log, int logNum, struct stateSet * sip) {
     struct stat sb;
-    time_t nowSecs = time(NULL);
+    logState * state = NULL;
+    struct tm now = *localtime(&nowSecs);
+    
+    message(MESS_DEBUG, "considering log %s\n", log->files[logNum]);
+    
+    if (stat(log->files[logNum], &sb)) {
+        if ((log->flags & LOG_FLAG_MISSINGOK) && (errno == ENOENT)) {
+            message(MESS_DEBUG, "  log %s does not exist -- skipping\n", 
+                    log->files[logNum]);
+            return 0;
+        }
+        message(MESS_ERROR, "stat of %s failed: %s\n", log->files[logNum],
+                strerror(errno));
+        return 1;
+    }
+
+    state = findState(log->files[logNum], sip);
+    state->doRotate = 0;
+    state->sb = sb;
+
+    if (log->criterium == ROT_SIZE) {
+        state->doRotate = (sb.st_size >= log->threshhold);
+    } else if (log->criterium == ROT_FORCE) {
+        /* user forced rotation of logs from command line */
+        state->doRotate = 1;
+    } else if (state->lastRotated.tm_year > now.tm_year || 
+	       (state->lastRotated.tm_year == now.tm_year && 
+		(state->lastRotated.tm_mon > now.tm_mon ||
+		 (state->lastRotated.tm_mon == now.tm_mon &&
+		  state->lastRotated.tm_mday > now.tm_mday)))) {
+        message(MESS_ERROR,
+		"log %s last rotated in the future -- rotation forced\n",
+		log->files[logNum]);
+        state->doRotate = 1;
+    } else if (state->lastRotated.tm_year != now.tm_year || 
+               state->lastRotated.tm_mon != now.tm_mon ||
+               state->lastRotated.tm_mday != now.tm_mday) {
+        switch (log->criterium) {
+          case ROT_WEEKLY:
+            /* rotate if:
+                  1) the current weekday is before the weekday of the
+                     last rotation
+                  2) more then a week has passed since the last
+                     rotation */
+            state->doRotate = ((now.tm_wday < state->lastRotated.tm_wday) ||
+			       ((mktime(&now) - mktime(&state->lastRotated)) >
+				(7 * 24 * 3600)));
+            break;
+          case ROT_MONTHLY:
+            /* rotate if the logs haven't been rotated this month or
+               this year */
+	      state->doRotate = ((now.tm_mon != state->lastRotated.tm_mon) ||
+				 (now.tm_year != state->lastRotated.tm_year));
+            break;
+          case ROT_DAYS:
+            /* FIXME: only days=1 is implemented!! */
+	      state->doRotate = 1;
+	      break;
+	default:
+            /* ack! */
+            state->doRotate = 0;
+            break;
+        }
+    }
+
+    /* The notifempty flag overrides the normal criteria */
+    if (!(log->flags & LOG_FLAG_IFEMPTY) && !sb.st_size)
+        state->doRotate = 0;
+    
+    if (state->doRotate) {
+        message(MESS_DEBUG, "  log needs rotating\n");
+    } else {
+        message(MESS_DEBUG, "  log does not need rotating\n");
+    }
+
+    return 0;
+}
+
+int rotateSingleLog(logInfo * log, int logNum, logState * state) {
     struct tm now = *localtime(&nowSecs);
     char * oldName, * newName = NULL;
     char * disposeName;
@@ -184,472 +450,459 @@
     char * compext = "";
     char * fileext = "";
     int hasErrors = 0;
-    int doRotate = 0;
     int i;
     int fd;
-    logState * state = NULL;
     uid_t createUid;
     gid_t createGid;
     mode_t createMode;
     char * baseName;
     char * dirName;
     char * firstRotated;
+    size_t alloc_size;
     int rotateCount = log->rotateCount ? log->rotateCount : 1;
-
-    /* Logs with rotateCounts of 0 are rotated to .1, then removed. This
+    int logStart = (log->logStart == -1) ? 1 : log->logStart;
+
+    if (!state->doRotate) return 0;
+
+    /* Logs with rotateCounts of 0 are rotated once, then removed. This
        lets scripts run properly, and everything gets mailed properly. */
 
-    message(MESS_DEBUG, "rotating file %s\n", log->files[logNum]);
-
+    message(MESS_DEBUG, "rotating log %s, log->rotateCount is %d\n", log->files[logNum], log->rotateCount);
+    
     if (log->flags & LOG_FLAG_COMPRESS) compext = log->compress_ext;
     
-    if (stat(log->files[logNum], &sb)) {
-	if ((log->flags & LOG_FLAG_MISSINGOK) && (errno == ENOENT)) {
-	    message(MESS_DEBUG, "file %s does not exist -- skipping\n", 
-		    log->files[logNum]);
-	    return 0;
-	}
-	fprintf(errorFile, "stat of %s failed: %s\n", log->files[logNum],
+    state->lastRotated = now;
+    
+    if (log->oldDir) {
+	if (log->oldDir[0] != '/') {
+	    char *ld = ourDirName(log->files[logNum]);
+	    dirName = malloc(strlen(ld) + strlen(log->oldDir) + 2);
+	    sprintf(dirName, "%s/%s", ld, log->oldDir);
+	    free(ld);
+	} else
+	  dirName = strdup(log->oldDir);
+    } else
+        dirName = ourDirName(log->files[logNum]);
+
+    baseName = strdup(ourBaseName(log->files[logNum]));
+
+    alloc_size = strlen(dirName) + strlen(baseName) + 
+                 strlen(log->files[logNum]) + strlen(fileext) +
+                 strlen(compext) + 10;
+    
+    oldName = alloca(alloc_size);
+    newName = alloca(alloc_size);
+    disposeName = alloca(alloc_size);
+    
+    if (log->extension &&
+	strncmp(&baseName[strlen(baseName)-strlen(log->extension)],
+		log->extension, strlen(log->extension)) == 0) {
+        char *tempstr;
+	
+        fileext = log->extension;
+        tempstr = calloc(strlen(baseName)-strlen(log->extension)+1, sizeof(char));
+        strncat(tempstr, baseName,
+                strlen(baseName)-strlen(log->extension));
+        free(baseName);
+        baseName = tempstr;
+    }
+    
+    /* First compress the previous log when necessary */
+    if (log->flags & LOG_FLAG_COMPRESS &&
+        log->flags & LOG_FLAG_DELAYCOMPRESS) {
+        struct stat sbprev;
+	
+        sprintf(oldName, "%s/%s.%d%s", dirName, baseName, logStart, fileext);
+        if (stat(oldName, &sbprev)) {
+            message(MESS_DEBUG, "previous log %s does not exist\n",
+		    oldName);
+        } else {
+	    hasErrors = compressLogFile(oldName, log);
+	}
+    }
+    
+    sprintf(oldName, "%s/%s.%d%s%s", dirName, baseName,
+            logStart + rotateCount, fileext, compext);
+    strcpy(newName, oldName);
+    
+    strcpy(disposeName, oldName);
+    
+    firstRotated = alloca(strlen(dirName) + strlen(baseName) +
+                          strlen(fileext) + strlen(compext) + 30);
+    sprintf(firstRotated, "%s/%s.%d%s%s", dirName, baseName,
+            logStart, fileext, 
+	    (log->flags & LOG_FLAG_DELAYCOMPRESS) ? "" : compext);
+    
+#ifdef WITH_SELINUX
+    if ((selinux_enabled=(is_selinux_enabled()>0))) {
+      security_context_t oldContext=NULL;
+      if (getfilecon(log->files[logNum], &oldContext)>0) {
+	if (getfscreatecon(&prev_context) < 0) {
+	  message(MESS_ERROR, "error getting default context: %s\n", 
+		  strerror(errno));
+	  freecon(oldContext);
+	  return 1;
+	}
+	if (setfscreatecon(oldContext) < 0) {
+	  message(MESS_ERROR, "error setting file context %s to %s: %s\n", 
+		  log->files[logNum], oldContext,strerror(errno));
+	  freecon(oldContext);
+	  return 1;
+	}
+	freecon(oldContext);
+      } else {
+	message(MESS_ERROR, "error getting file context %s: %s\n", 
+		log->files[logNum], 
 		strerror(errno));
-	hasErrors = 1;
-    }
-
+	return 1;
+      }
+    }
+#endif
+    for (i = rotateCount + logStart - 1; (i >= 0) && !hasErrors; i--) {
+        tmp = newName;
+        newName = oldName;
+        oldName = tmp;
+        sprintf(oldName, "%s/%s.%d%s%s", dirName, baseName, i,
+                fileext, compext);
+	
+        message(MESS_DEBUG, "renaming %s to %s (rotatecount %d, logstart %d, i %d), \n", oldName, newName,
+		rotateCount, logStart, i);
+	
+        if (!debug && rename(oldName, newName)) {
+            if (errno == ENOENT) {
+                message(MESS_DEBUG, "old log %s does not exist\n",
+                        oldName);
+            } else {
+                message(MESS_ERROR, "error renaming %s to %s: %s\n",
+                        oldName, newName, strerror(errno));
+                hasErrors = 1;
+	    }
+	}
+    }
+    
+    finalName = oldName;
+    
+    /* note: the gzip extension is *not* used here! */
+    sprintf(finalName, "%s/%s.%d%s", dirName, baseName, logStart, fileext);
+    
+    /* if the last rotation doesn't exist, that's okay */
+    if (!debug && access(disposeName, F_OK)) {
+        message(MESS_DEBUG, "log %s doesn't exist -- won't try to "
+                "dispose of it\n", disposeName);
+        disposeName = NULL;
+    } 
+    
+    free(dirName);
+    free(baseName);
+    
     if (!hasErrors) {
-	state = findState(log->files[logNum], statesPtr, numStatesPtr);
-
-	if (log->criterium == ROT_SIZE) {
-	    doRotate = (sb.st_size >= log->threshhold);
-	}
-	else if (log->criterium == ROT_FORCE) {
-	    /* user forced rotation of logs from command line */
-	    doRotate = 1;
-	} else if (state->lastRotated.tm_year > now.tm_year || 
-		      (state->lastRotated.tm_year == now.tm_year && 
-			  (state->lastRotated.tm_mon > now.tm_mon ||
-			      (state->lastRotated.tm_mon == now.tm_mon &&
-			       state->lastRotated.tm_mday > now.tm_mday)
-			  )
-		      )
-		  ) {
-	    message(MESS_ERROR,
-	    "file %s last rotated in the future -- rotation forced\n",
-	    log->files[logNum]);
-	    doRotate = 1;
-	} else if (state->lastRotated.tm_year != now.tm_year || 
-		   state->lastRotated.tm_mon != now.tm_mon ||
-		   state->lastRotated.tm_mday != now.tm_mday) {
-	    switch (log->criterium) {
-	      case ROT_WEEKLY:
-		/* rotate if:
-		      1) the current weekday is before the weekday of the
-			 last rotation
-		      2) more then a week has passed since the last
-			 rotation 
-		*/
-		doRotate = ((now.tm_wday < state->lastRotated.tm_wday) ||
-			    ((mktime(&now) - mktime(&state->lastRotated)) >
-			       (7 * 24 * 3600)));
-		break;
-	      case ROT_MONTHLY:
-		/* rotate if the logs haven't been rotated this month or
-		   this year */
-		doRotate = ((now.tm_mon != state->lastRotated.tm_mon) ||
-			    (now.tm_year != state->lastRotated.tm_year));
-		break;
-	      case ROT_DAYS:
-		/* FIXME: only days=1 is implemented!! */
-		doRotate = 1;
-		break;
-	      default:
-		/* ack! */
-		doRotate = 0;
-		break;
-	    }
-	} else {
-	    doRotate = 0;
-	}
-    }
-
-    /* The notifempty flag overrides the normal criteria */
-    if (!(log->flags & LOG_FLAG_IFEMPTY) && !sb.st_size)
-	doRotate = 0;
-
-    if (!hasErrors && doRotate) {
-	message(MESS_DEBUG, "log needs rotating\n");
-
-	state->lastRotated = now;
-
-	if (log->oldDir)
-	    dirName = strdup(log->oldDir);
-	else
-	    dirName = ourDirName(log->files[logNum]);
-	baseName = strdup(ourBaseName(log->files[logNum]));
-
-	oldName = alloca(strlen(dirName) + strlen(baseName) + 
-			    strlen(log->files[logNum]) + 10);
-	newName = alloca(strlen(dirName) + strlen(baseName) + 
-			    strlen(log->files[logNum]) + 10);
-	disposeName = alloca(strlen(dirName) + strlen(baseName) + 
-			    strlen(log->files[logNum]) + 10);
-
-
-	if (log->extension)
-	    fileext = log->extension;
-
-	if (log->extension &&
-		strncmp(&baseName[strlen(baseName)-strlen(log->extension)],
-		log->extension, strlen(log->extension)) == 0) {
-	    char *tempstr;
-	    tempstr = calloc(strlen(baseName)-strlen(log->extension)+1, sizeof(char));
-	    strncat(tempstr, baseName,
-		    strlen(baseName)-strlen(log->extension));
-	    free(baseName);
-	    baseName = tempstr;
-	}
-
-	/* First compress the previous log when necessary */
-	if (log->flags & LOG_FLAG_COMPRESS &&
-		    log->flags & LOG_FLAG_DELAYCOMPRESS) {
-	    struct stat sbprev;
-
-	    sprintf(oldName, "%s/%s.1%s", dirName, baseName, fileext);
-	    if (stat(oldName, &sbprev)) {
-		message(MESS_DEBUG, "previous log %s does not exist\n",
-				    oldName);
-	    } else {
-		char * command;
-
-		command = alloca(strlen(oldName) +
-				    strlen(log->compress_prog) + 1 + strlen(log->compress_options) + 20);
-		sprintf(command, "%s %s '%s'", log->compress_prog, log->compress_options, oldName);
-		message(MESS_DEBUG, "compressing previous log with: %s\n",
-				    command);
-		if (!debug && system(command)) {
-		    fprintf(errorFile,
-			"failed to compress previous log %s\n", oldName);
-		    hasErrors = 1;
-		}
-	    }
-	}
-
-	sprintf(oldName, "%s/%s.%d%s%s", dirName, baseName,
-		rotateCount + 1, fileext, compext);
-
-	strcpy(disposeName, oldName);
-
-	firstRotated = alloca(strlen(dirName) + strlen(baseName) +
-			      strlen(fileext) + strlen(compext) + 30);
-	sprintf(firstRotated, "%s/%s.1%s%s", dirName, baseName,
-		fileext, compext);
-
-	for (i = rotateCount; i && !hasErrors; i--) {
-	    tmp = newName;
-	    newName = oldName;
-	    oldName = tmp;
-	    sprintf(oldName, "%s/%s.%d%s%s", dirName, baseName, i,
-		    fileext, compext);
-
-	    message(MESS_DEBUG, "renaming %s to %s\n", oldName, newName);
-
-	    if (!debug && rename(oldName, newName)) {
-		if (errno == ENOENT) {
-		    message(MESS_DEBUG, "old log %s does not exist\n",
-			    oldName);
-		} else {
-		    fprintf(errorFile, "error renaming %s to %s: %s\n",
-			    oldName, newName, strerror(errno));
-		    hasErrors = 1;
-		}
-	    }
-	}
-
-	finalName = oldName;
-
-	/* note: the gzip extension is *not* used here! */
-	sprintf(finalName, "%s/%s.1%s", dirName, baseName, fileext);
-
-	/* if the last rotation doesn't exist, that's okay */
-	if (!debug && access(disposeName, F_OK)) {
-	    message(MESS_DEBUG, "file %s doesn't exist -- won't try "
-		    "dispose of it\n", disposeName);
-	    disposeName = NULL;
-	} 
-
-	free(dirName);
-	free(baseName);
-
-	if (!hasErrors) {
-	    if (log->pre && !(log->flags & LOG_FLAG_SHAREDSCRIPTS)) {
-		message(MESS_DEBUG, "running prerotate script\n");
-		if (runScript(log->files[logNum], log->pre)) {
-		    fprintf(errorFile, "error running prerotate script -- 
-				leaving old log in place\n");
-		    hasErrors = 1;
-		}
-	    }
-
-	    if (!(log->flags & LOG_FLAG_COPYTRUNCATE)) {
-		message(MESS_DEBUG, "renaming %s to %s\n", log->files[logNum], 
+        if (log->pre && !(log->flags & LOG_FLAG_SHAREDSCRIPTS)) {
+            message(MESS_DEBUG, "running prerotate script\n");
+            if (runScript(log->files[logNum], log->pre)) {
+                message(MESS_ERROR, "error running prerotate script, "
+			"leaving old log in place\n");
+                hasErrors = 1;
+	    }
+        }
+	
+        if (!(log->flags & (LOG_FLAG_COPYTRUNCATE|LOG_FLAG_COPY))) {
+            message(MESS_DEBUG, "renaming %s to %s\n", log->files[logNum], 
 		    finalName);
-
-		if (!debug && !hasErrors &&
-			rename(log->files[logNum], finalName)) {
-		    fprintf(errorFile, "failed to rename %s to %s: %s\n",
+	    
+            if (!debug && !hasErrors &&
+		rename(log->files[logNum], finalName)) {
+                message(MESS_ERROR, "failed to rename %s to %s: %s\n",
 			log->files[logNum], finalName, strerror(errno));
-		}
-	    }
-
-	    if (!hasErrors && log->flags & LOG_FLAG_CREATE &&
-			!(log->flags & LOG_FLAG_COPYTRUNCATE)) {
-		if (log->createUid == NO_UID)
-		    createUid = sb.st_uid;
-		else
-		    createUid = log->createUid;
-	    
-		if (log->createGid == NO_GID)
-		    createGid = sb.st_gid;
-		else
-		    createGid = log->createGid;
-	    
-		if (log->createMode == NO_MODE)
-		    createMode = sb.st_mode & 0777;
-		else
-		    createMode = log->createMode;
-	    
-		message(MESS_DEBUG, "creating new log mode = 0%o uid = %d "
-			"gid = %d\n", createMode, createUid, createGid);
-
-		if (!debug) {
-		    fd = open(log->files[logNum], O_CREAT | O_RDWR, createMode);
-		    if (fd < 0) {
-			message(MESS_ERROR, "error creating %s: %s\n", 
-				log->files[logNum], strerror(errno));
-			hasErrors = 1;
-		    } else {
-			if (fchmod(fd, createMode)) {
-			    message(MESS_ERROR, "error setting mode of "
-				    "%s: %s\n", log->files[logNum], 
-				     strerror(errno));
-			    hasErrors = 1;
-			}
-			if (fchown(fd, createUid, createGid)) {
-			    message(MESS_ERROR, "error setting owner of "
-				    "%s: %s\n", log->files[logNum], 
-				     strerror(errno));
-			    hasErrors = 1;
-			}
-
-			close(fd);
+	    }
+
+	    if (!log->rotateCount) {
+	      disposeName = alloca(strlen(dirName) + strlen(baseName) + 
+				   strlen(log->files[logNum]) + 10);
+	      sprintf(disposeName, "%s%s", finalName, (log->compress_ext && (log->flags & LOG_FLAG_COMPRESS))?log->compress_ext:"");
+	      message(MESS_DEBUG, "disposeName will be %s\n", disposeName);
+	    }
+        }
+	
+        if (!hasErrors && log->flags & LOG_FLAG_CREATE &&
+            !(log->flags & (LOG_FLAG_COPYTRUNCATE|LOG_FLAG_COPY))) {
+            if (log->createUid == NO_UID)
+                createUid = state->sb.st_uid;
+            else
+                createUid = log->createUid;
+	    
+            if (log->createGid == NO_GID)
+                createGid = state->sb.st_gid;
+            else
+                createGid = log->createGid;
+	    
+            if (log->createMode == NO_MODE)
+                createMode = state->sb.st_mode & 0777;
+            else
+                createMode = log->createMode;
+	    
+            message(MESS_DEBUG, "creating new log mode = 0%o uid = %d "
+                    "gid = %d\n", (unsigned int)createMode, (int)createUid, (int)createGid);
+	    
+            if (!debug) {
+                fd = open(log->files[logNum], O_CREAT | O_RDWR, createMode);
+                if (fd < 0) {
+                    message(MESS_ERROR, "error creating %s: %s\n", 
+                            log->files[logNum], strerror(errno));
+                    hasErrors = 1;
+                } else {
+                    if (fchmod(fd, (S_IRUSR | S_IWUSR) & createMode)) {
+                        message(MESS_ERROR, "error setting mode of "
+                                "%s: %s\n", log->files[logNum], 
+				strerror(errno));
+                        hasErrors = 1;
+                    }
+                    if (fchown(fd, createUid, createGid)) {
+                        message(MESS_ERROR, "error setting owner of "
+                                "%s: %s\n", log->files[logNum], 
+				strerror(errno));
+                        hasErrors = 1;
 		    }
-		}
-	    }
-
-	    if (!hasErrors && log->flags & LOG_FLAG_COPYTRUNCATE) {
-		hasErrors = copyTruncate(log->files[logNum], finalName, &sb);
-
-	    }
-
-	    if (!hasErrors && log->post && 
-		    !(log->flags & LOG_FLAG_SHAREDSCRIPTS)) {
-		message(MESS_DEBUG, "running postrotate script\n");
-		if (runScript(log->files[logNum], log->post)) {
-		    fprintf(errorFile, "error running postrotate script\n");
-		    hasErrors = 1;
-		}
-	    }
-
-	    if (!hasErrors && !log->rotateCount) {
-		message(MESS_DEBUG, "removing rotated log (rotateCount == 0)");
-		if (unlink(finalName)) {
-		    fprintf(errorFile, "Failed to remove old log %s: %s\n",
-				finalName, strerror(errno));
-		    hasErrors = 1;
-		}
-	    }
-
-	    if (!hasErrors && log->rotateCount && 
-			(log->flags & LOG_FLAG_COMPRESS) &&
-			!(log->flags & LOG_FLAG_DELAYCOMPRESS)) {
-		char * command;
-
-		command = alloca(strlen(finalName) + strlen(log->compress_prog) + 1 + strlen(log->compress_options) + 20);
-
-		sprintf(command, "%s %s '%s'", log->compress_prog, log->compress_options, finalName);
-		message(MESS_DEBUG, "compressing new log with: %s\n", command);
-		if (!debug && system(command)) {
-		    fprintf(errorFile, "failed to compress log %s\n", 
-				finalName);
-		    hasErrors = 1;
-		}
-	    }
-
-	    if (!hasErrors && log->logAddress) {
-		char * command;
-		char * mailFilename;
-
-		if (log->flags & LOG_FLAG_MAILFIRST)
-		    mailFilename = firstRotated;
-		else
-		    mailFilename = disposeName;
-
-		if (mailFilename) {
-		    command = alloca(strlen(mailFilename) + 100 + 
-				     strlen(log->uncompress_prog));
-
-		    if ((log->flags & LOG_FLAG_COMPRESS) &&
-			    !((log->flags & LOG_FLAG_DELAYCOMPRESS) &&
-			    (log->flags & LOG_FLAG_MAILFIRST)))
-			sprintf(command, "%s < %s | %s '%s' %s", 
-				    log->uncompress_prog, mailFilename, mailCommand,
-				    log->files[logNum],
-				    log->logAddress);
-		    else
-			sprintf(command, "%s '%s' %s < %s", mailCommand, 
-				    mailFilename, log->logAddress, mailFilename);
-
-		    message(MESS_DEBUG, "executing: \"%s\"\n", command);
-
-		    if (!debug && system(command)) {
-			sprintf(newName, "%s.%d", log->files[logNum], getpid());
-			fprintf(errorFile, "Failed to mail %s to %s!\n",
-				mailFilename, log->logAddress);
-
-			hasErrors = 1;
-		    } 
-		}
-	    }
-
-	    if (!hasErrors && disposeName) {
-		message(MESS_DEBUG, "removing old log %s\n", disposeName);
-
-		if (!debug && unlink(disposeName)) {
-		    fprintf(errorFile, "Failed to remove old log %s: %s\n",
-				disposeName, strerror(errno));
-		    hasErrors = 1;
-		}
-	    }
-
-	}
-    } else if (!doRotate) {
-	message(MESS_DEBUG, "log does not need rotating\n");
+	            if (fchmod(fd, createMode)) {
+                        message(MESS_ERROR, "error setting mode of "
+                                "%s: %s\n", log->files[logNum], 
+				strerror(errno));
+                        hasErrors = 1;
+                    }
+
+                    close(fd);
+		}
+	    }
+        }
+	
+        if (!hasErrors && log->flags & (LOG_FLAG_COPYTRUNCATE|LOG_FLAG_COPY))
+            hasErrors = copyTruncate(log->files[logNum], finalName,
+                                     &state->sb, log->flags);
+	
+        if (!hasErrors && log->post && 
+	    !(log->flags & LOG_FLAG_SHAREDSCRIPTS)) {
+            message(MESS_DEBUG, "running postrotate script\n");
+            if (runScript(log->files[logNum], log->post)) {
+                message(MESS_ERROR, "error running postrotate script\n");
+                hasErrors = 1;
+	    }
+        }
+
+        if (!hasErrors && 
+	    (log->flags & LOG_FLAG_COMPRESS) &&
+	    !(log->flags & LOG_FLAG_DELAYCOMPRESS)) {
+	    hasErrors = compressLogFile(finalName, log);
+	}
+	
+        if (!hasErrors && log->logAddress) {
+            char * mailFilename;
+	    
+            if (log->flags & LOG_FLAG_MAILFIRST)
+                mailFilename = firstRotated;
+            else
+                mailFilename = disposeName;
+
+            if (mailFilename) {
+		/* if the log is compressed (and we're not mailing a
+		   file whose compression has been delayed), we need
+		   to uncompress it */
+                if ((log->flags & LOG_FLAG_COMPRESS) &&
+		    !((log->flags & LOG_FLAG_DELAYCOMPRESS) &&
+		      (log->flags & LOG_FLAG_MAILFIRST))) {
+		    if (mailLog(mailFilename, mailCommand, 
+				log->uncompress_prog, log->logAddress, 
+				log->files[logNum])) 
+			hasErrors = 1;
+		} else {
+		    if (mailLog(mailFilename, mailCommand, NULL, 
+			        log->logAddress, mailFilename))
+			hasErrors = 1;
+		}
+	    }
+        }
+	
+        if (!hasErrors && disposeName) {
+            message(MESS_DEBUG, "removing old log %s\n", disposeName);
+	    
+            if (!debug && unlink(disposeName)) {
+                message(MESS_ERROR, "Failed to remove old log %s: %s\n",
+			disposeName, strerror(errno));
+                hasErrors = 1;
+	    }
+	}
     }
-
+    
+#ifdef WITH_SELINUX
+	if (selinux_enabled) {
+	  setfscreatecon(prev_context);
+	  if (prev_context!= NULL) {
+	    freecon(prev_context);
+	    prev_context=NULL;
+	  }
+	}
+#endif
     return hasErrors;
 }
 
-int rotateLogSet(logInfo * log, logState ** statesPtr, int * numStatesPtr, 
-		 int force) {
+int rotateLogSet(logInfo * log, struct stateSet * sip, int force) {
     int i;
     int hasErrors = 0;
+    int numRotated = 0;
+    logState * state;
 
     if (force)
 	log->criterium = ROT_FORCE;
-
-    message(MESS_DEBUG, "rotating pattern: %s ", log->pattern);
+    
+    message(MESS_DEBUG, "\nrotating pattern: %s ", log->pattern);
     switch (log->criterium) {
-      case ROT_DAYS:
+    case ROT_DAYS:
 	message(MESS_DEBUG, "after %d days ", log->threshhold);
 	break;
-      case ROT_WEEKLY:
+    case ROT_WEEKLY:
 	message(MESS_DEBUG, "weekly ");
 	break;
-      case ROT_MONTHLY:
+    case ROT_MONTHLY:
 	message(MESS_DEBUG, "monthly ");
 	break;
-      case ROT_SIZE:
+    case ROT_SIZE:
 	message(MESS_DEBUG, "%d bytes ", log->threshhold);
 	break;
-      case ROT_FORCE:
+    case ROT_FORCE:
 	message(MESS_DEBUG, "forced from command line ");
 	break;
     }
-
+    
     if (log->rotateCount) 
 	message(MESS_DEBUG, "(%d rotations)\n", log->rotateCount);
     else
 	message(MESS_DEBUG, "(no old logs will be kept)\n");
-
+    
     if (log->oldDir) 
-	message(MESS_DEBUG, "olddir is %s ", log->oldDir);
-
+        message(MESS_DEBUG, "olddir is %s, ", log->oldDir);
+    
     if (log->flags & LOG_FLAG_IFEMPTY) 
-	message(MESS_DEBUG, "empty log files are rotated ");
+        message(MESS_DEBUG, "empty log files are rotated, ");
     else
-	message(MESS_DEBUG, "empty log files are not rotated ");
-
+        message(MESS_DEBUG, "empty log files are not rotated, ");
+    
     if (log->logAddress) {
 	message(MESS_DEBUG, "old logs mailed to %s\n", log->logAddress);
     } else {
 	message(MESS_DEBUG, "old logs are removed\n");
     }
+    
+    for (i = 0; i < log->numFiles; i++) {
+        hasErrors |= findNeedRotating(log, i, sip);
+	
+        /* sure is a lot of findStating going on .. */
+        if ((findState(log->files[i], sip))->doRotate)
+            numRotated++;
+    }
+    
+    if (log->first) {
+        if (!numRotated) {
+            message(MESS_DEBUG, "not running first action script, "
+                    "since no logs will be rotated\n");
+        } else {
+            message(MESS_DEBUG, "running first action script\n");
+            if (runScript(log->pattern, log->first)) {
+                message(MESS_ERROR, "error running first action script "
+                        "for %s\n", log->pattern);
+                hasErrors = 1;
+            }
+	}
+    }
 
     if (log->pre && (log->flags & LOG_FLAG_SHAREDSCRIPTS)) {
-	message(MESS_DEBUG, "running shared prerotate script\n");
-	if (runScript(log->pattern, log->pre)) {
-	    fprintf(stderr, "error running shared prerotate script for %s-- 
-			leaving old logs in place\n", log->pattern);
-	    hasErrors = 1;
+        if (!numRotated) {
+            message(MESS_DEBUG, "not running shared prerotate script, "
+                    "since no logs will be rotated\n");
+        } else {
+            message(MESS_DEBUG, "running shared prerotate script\n");
+            if (runScript(log->pattern, log->pre)) {
+                message(MESS_ERROR, "error running shared prerotate script "
+                        "for %s\n", log->pattern);
+                hasErrors = 1;
+            }
 	}
     }
-
-    for (i = 0; i < log->numFiles; i++)
-	hasErrors |= rotateSingleLog(log, i, statesPtr, numStatesPtr, 
-					stderr);
-
+    
+    /* should there be an if(!hasErrors) here? */
+    for (i = 0; i < log->numFiles; i++) {
+	state = findState(log->files[i], sip);
+	
+	hasErrors |= rotateSingleLog(log, i, state);
+    }
+    
     if (log->post && (log->flags & LOG_FLAG_SHAREDSCRIPTS)) {
-	message(MESS_DEBUG, "running shared postrotate script\n");
-	if (runScript(log->pattern, log->post)) {
-	    fprintf(stderr, 
-	       "error running shared postrotate script for %s\n", log->pattern);
-	    hasErrors = 1;
-	}
-    }
-
+        if (!numRotated) {
+            message(MESS_DEBUG, "not running shared postrotate script, "
+                    "since no logs were rotated\n");
+        } else {
+            message(MESS_DEBUG, "running shared postrotate script\n");
+            if (runScript(log->pattern, log->post)) {
+                message(MESS_ERROR, "error running shared postrotate script "
+                        "for %s\n", log->pattern);
+                hasErrors = 1;
+            }
+	}
+    }
+    
+    if (log->last) {
+        if (!numRotated) {
+            message(MESS_DEBUG, "not running last action script, "
+                    "since no logs will be rotated\n");
+        } else {
+            message(MESS_DEBUG, "running last action script\n");
+            if (runScript(log->pattern, log->last)) {
+                message(MESS_ERROR, "error running last action script "
+                        "for %s\n", log->pattern);
+                hasErrors = 1;
+            }
+	}
+    }
+    
     return hasErrors;
 }
 
-static int writeState(char * stateFilename, logState * states, 
-		      int numStates) {
+static int writeState(char * stateFilename, struct stateSet si) {
     FILE * f;
     char * chptr;
     int i;
-
+    
     f = fopen(stateFilename, "w");
+    
     if (!f) {
 	message(MESS_ERROR, "error creating state file %s: %s\n", 
-		    stateFilename, strerror(errno));
+		stateFilename, strerror(errno));
 	return 1;
     }
-
+    
     fprintf(f, "logrotate state -- version 2\n");
-
-    for (i = 0; i < numStates; i++) {
+    
+    for (i = 0; i < si.numStates; i++) {
 	fputc('"', f);
-	for (chptr = states[i].fn; *chptr; chptr++) {
+	for (chptr = si.states[i].fn; *chptr; chptr++) {
 	    switch (*chptr) {
-	      case '\\':
-	      case '\'':
-	      case '"':
-	          fputc('\\', f);
+	    case '"':
+		fputc('\\', f);
 	    }
-
+	    
 	    fputc(*chptr, f);
 	}
-
+	
 	fputc('"', f);
 	fprintf(f, " %d-%d-%d\n", 
-		states[i].lastRotated.tm_year + 1900,
-		states[i].lastRotated.tm_mon + 1,
-		states[i].lastRotated.tm_mday);
+		si.states[i].lastRotated.tm_year + 1900,
+		si.states[i].lastRotated.tm_mon + 1,
+		si.states[i].lastRotated.tm_mday);
     }
-
+    
     fclose(f);
-
+    
     return 0;
 }
 
-static int readState(char * stateFilename, logState ** statesPtr, 
-		     int * numStatesPtr) {
+static int readState(char * stateFilename, struct stateSet * sip) {
     FILE * f;
     char buf[1024];
     const char ** argv;
@@ -665,94 +918,94 @@
     error = stat(stateFilename, &f_stat);
 
     if ((error && errno == ENOENT) ||
-        (!error && f_stat.st_size == 0)) {
+	(!error && f_stat.st_size == 0)) {
 	/* create the file before continuing to ensure we have write
 	   access to the file */
 	f = fopen(stateFilename, "w");
 	if (!f) {
 	    message(MESS_ERROR, "error creating state file %s: %s\n", 
-			stateFilename, strerror(errno));
+		    stateFilename, strerror(errno));
 	    return 1;
 	}
 	fprintf(f, "logrotate state -- version 2\n");
 	fclose(f);
 	return 0;
     } else if (error) {
-	message(MESS_ERROR, "error using stat() on state file %s: %s\n", 
-		    stateFilename, strerror(errno));
+	message(MESS_ERROR, "error stat()ing state file %s: %s\n", 
+		stateFilename, strerror(errno));
 	return 1;
     }
 
     f = fopen(stateFilename, "r");
     if (!f) {
-	message(MESS_ERROR, "error opening state file %s: %s\n", 
+	message(MESS_ERROR, "error opening state file %s: %s\n",
 		stateFilename, strerror(errno));
 	return 1;
     }
-
+    
     if (!fgets(buf, sizeof(buf) - 1, f)) {
 	message(MESS_ERROR, "error reading top line of %s\n", stateFilename);
 	fclose(f);
 	return 1;
     }
-
+    
     if (strcmp(buf, "logrotate state -- version 1\n") &&
-           strcmp(buf, "logrotate state -- version 2\n")) {
+	strcmp(buf, "logrotate state -- version 2\n")) {
 	fclose(f);
 	message(MESS_ERROR, "bad top line in state file %s\n", stateFilename);
 	return 1;
     }
-
+    
     line++;
-
+    
     while (fgets(buf, sizeof(buf) - 1, f)) {
 	line++;
 	i = strlen(buf);
 	if (buf[i - 1] != '\n') {
 	    message(MESS_ERROR, "line too long in state file %s\n", 
-			stateFilename);
+		    stateFilename);
 	    fclose(f);
 	    return 1;
 	}
-
+	
 	buf[i - 1] = '\0';
-
+	
 	if (i == 1) continue;
-
+	
 	if (poptParseArgvString(buf, &argc, &argv) || (argc != 2) ||
-		(sscanf(argv[1], "%d-%d-%d", &year, &month, &day) != 3)) {
+	    (sscanf(argv[1], "%d-%d-%d", &year, &month, &day) != 3)) {
 	    message(MESS_ERROR, "bad line %d in state file %s\n", 
 		    line, stateFilename);
 	    fclose(f);
 	    return 1;
 	}
-
+	
 	/* Hack to hide earlier bug */
-	if ((year != 1900) && (year < 1970 || year > 2100)) {
+	if ((year != 1900) && (year < 1996 || year > 2100)) {
 	    message(MESS_ERROR, "bad year %d for file %s in state file %s\n",
-			year, argv[0], stateFilename);
+		    year, argv[0], stateFilename);
 	    fclose(f);
 	    return 1;
 	}
-
+	
 	if (month < 1 || month > 12) {
 	    message(MESS_ERROR, "bad month %d for file %s in state file %s\n",
-			month, argv[0], stateFilename);
+		    month, argv[0], stateFilename);
 	    fclose(f);
 	    return 1;
 	}
-
+	
 	/* 0 to hide earlier bug */
 	if (day < 0 || day > 31) {
 	    message(MESS_ERROR, "bad day %d for file %s in state file %s\n",
-			day, argv[0], stateFilename);
+		    day, argv[0], stateFilename);
 	    fclose(f);
 	    return 1;
 	}
 
 	year -= 1900, month -= 1;
 
-	st = findState(argv[0], statesPtr, numStatesPtr);
+	st = findState(argv[0], sip);
 
 	st->lastRotated.tm_mon = month;
 	st->lastRotated.tm_mday = day;
@@ -774,17 +1027,20 @@
 int main(int argc, const char ** argv) {
     logInfo defConfig = { NULL, NULL, 0, NULL, ROT_SIZE, 
 			  /* threshHold */ 1024 * 1024, 0,
+			  /* log start */ -1,
 			  /* pre, post */ NULL, NULL,
+			  /* first, last */ NULL, NULL,
 			  /* logAddress */ NULL, 
 			  /* extension */ NULL, 
 			  /* compression */ COMPRESS_COMMAND,
-			  UNCOMPRESS_COMMAND, COMPRESS_OPTIONS, COMPRESS_EXT,
+			  UNCOMPRESS_COMMAND, COMPRESS_EXT,
+			  /* rotate pattern */ NULL,
 			  /* flags */ LOG_FLAG_IFEMPTY,
 			  /* createMode */ NO_MODE, NO_UID, NO_GID };
-    int numLogs = 0, numStates = 0;
+    int numLogs = 0;
     int force = 0;
     logInfo * logs = NULL;
-    logState * states = NULL;
+    struct stateSet si = { NULL, 0 };
     char * stateFile = STATEFILE;
     int i;
     int rc = 0;
@@ -793,17 +1049,18 @@
     poptContext optCon;
     struct poptOption options[] = {
 	{ "debug", 'd', 0, 0, 'd', 
-		"Don't do anything, just test (implies -v)" },
+	  "Don't do anything, just test (implies -v)" },
 	{ "force", 'f', 0 , &force, 0, "Force file rotation" },
 	{ "mail", 'm', POPT_ARG_STRING, &mailCommand, 0, 
-		"Command to use to mail logs", "command" },
+	  "Command to send mail (instead of `" DEFAULT_MAIL_COMMAND "')",
+	  "command" },
 	{ "state", 's', POPT_ARG_STRING, &stateFile, 0, "Path of state file",
-		"statefile" },
+	  "statefile" },
 	{ "verbose", 'v', 0, 0, 'v', "Display messages during rotation" },
 	POPT_AUTOHELP
 	{ 0, 0, 0, 0, 0 } 
     };
-
+    
     logSetLevel(MESS_NORMAL);
 
     optCon = poptGetContext("logrotate", argc, argv, options,0);
@@ -812,10 +1069,10 @@
 
     while ((arg = poptGetNextOpt(optCon)) >= 0) {
         switch (arg) {
-	  case 'd':
+	case 'd':
 	    debug = 1;
 	    /* fallthrough */
-	  case 'v':
+	case 'v':
 	    logSetLevel(MESS_DEBUG);
 	    break;
 	}
@@ -828,12 +1085,12 @@
 	return 2;
     }
 
-    files = poptGetArgs(optCon);
+    files = poptGetArgs((poptContext) optCon);
     if (!files) {
 	fprintf(stderr, "logrotate " VERSION 
-		    " - Copyright (C) 1995-2001 Red Hat, Inc.\n");
+		" - Copyright (C) 1995-2001 Red Hat, Inc.\n");
 	fprintf(stderr, "This may be freely redistributed under the terms of "
-		    "the GNU Public License\n\n");
+		"the GNU Public License\n\n");
 	poptPrintUsage(optCon, stderr, 0);
 	exit(1);
     }
@@ -844,17 +1101,19 @@
 	}
     }
 
-    if (readState(stateFile, &states, &numStates)) {
+    nowSecs = time(NULL);
+
+    if (readState(stateFile, &si)) {
 	exit(1);
     }
 
-    message(MESS_DEBUG, "Handling %d logs\n", numLogs);
+    message(MESS_DEBUG, "\nHandling %d logs\n", numLogs);
 
     for (i = 0; i < numLogs; i++) {
-	rc |= rotateLogSet(logs + i, &states, &numStates, force);
+	rc |= rotateLogSet(logs + i, &si, force);
     }
 
-    if (!debug && writeState(stateFile, states, numStates)) {
+    if (!debug && writeState(stateFile, si)) {
 	exit(1);
     }
 

=== modified file 'logrotate.h'
--- logrotate.h	2002-04-23 23:16:43 +0000
+++ logrotate.h	2004-06-11 13:51:34 +0000
@@ -4,6 +4,8 @@
 #include <sys/types.h>
 #include <glob.h>
 
+#include "config.h"
+
 #define LOG_FLAG_COMPRESS	(1 << 0)
 #define LOG_FLAG_CREATE		(1 << 1)
 #define LOG_FLAG_IFEMPTY	(1 << 2)
@@ -12,17 +14,17 @@
 #define LOG_FLAG_MISSINGOK	(1 << 5)
 #define LOG_FLAG_MAILFIRST	(1 << 6)
 #define LOG_FLAG_SHAREDSCRIPTS	(1 << 7)
-
-#define DEFAULT_MAIL_COMMAND "/usr/bin/mail -s"
-#define COMPRESS_COMMAND "/bin/gzip"
-#define COMPRESS_OPTIONS "-9"
-#define COMPRESS_EXT ".gz"
-#define UNCOMPRESS_COMMAND "/bin/gunzip"
+#define LOG_FLAG_COPY		(1 << 8)
 
 #define NO_FORCE_ROTATE 0
 #define FORCE_ROTATE    1
 
-#define STATEFILE "/var/lib/logrotate/status"
+struct rotatePatternElement {
+    enum { RP_NONE = 0, RP_FILENAME, RP_STRING, RP_COUNT, RP_MONTH, RP_DAY, 
+	   RP_YEAR } type;
+    char * arg;
+    struct rotatePatternElement * next;
+};
 
 typedef struct {
     char * pattern;
@@ -32,17 +34,21 @@
     enum { ROT_DAYS, ROT_WEEKLY, ROT_MONTHLY, ROT_SIZE, ROT_FORCE } criterium;
     unsigned int threshhold;
     int rotateCount;
-    char * pre, * post;
+    int logStart;
+    char * pre, * post, * first, * last;
     char * logAddress;
     char * extension;
     char * compress_prog;
     char * uncompress_prog;
-    char * compress_options;
     char * compress_ext;
+    struct rotatePatternElement * rotatePattern;
     int flags;
     mode_t createMode;		/* if any/all of these are -1, we use the */
     uid_t createUid;		/* attributes from the log file just rotated */
     gid_t createGid;
+    /* these are at the end so they end up nil */
+    const char ** compress_options_list;
+    int compress_options_count;
 } logInfo;
 
 int readConfigPath(const char * path, logInfo * defConfig, 

=== modified file 'logrotate.spec'
--- logrotate.spec	2002-04-23 23:16:43 +0000
+++ logrotate.spec	2004-06-11 13:51:34 +0000
@@ -1,7 +1,11 @@
+%if %{?WITH_SELINUX:0}%{!?WITH_SELINUX:1}
+%define WITH_SELINUX 1
+%endif
+
 Summary: Rotates, compresses, removes and mails system log files.
 Name: logrotate
-Version: 3.5.9
-Release: 1
+Version: 3.7
+Release: 4
 License: GPL
 Group: System Environment/Base
 Source: logrotate-%{PACKAGE_VERSION}.tar.gz
@@ -22,15 +26,21 @@
 %setup
 
 %build
-make RPM_OPT_FLAGS="$RPM_OPT_FLAGS"
+make RPM_OPT_FLAGS="$RPM_OPT_FLAGS" \
+%if %{WITH_SELINUX}
+	WITH_SELINUX=yes
+%endif
 
 %install
+rm -rf $RPM_BUILD_ROOT
 make PREFIX=$RPM_BUILD_ROOT MANDIR=%{_mandir} install
 mkdir -p $RPM_BUILD_ROOT/etc/logrotate.d
 mkdir -p $RPM_BUILD_ROOT/etc/cron.daily
+mkdir -p $RPM_BUILD_ROOT/var/lib
 
 install -m 644 examples/logrotate-default $RPM_BUILD_ROOT/etc/logrotate.conf
 install -m 755 examples/logrotate.cron $RPM_BUILD_ROOT/etc/cron.daily/logrotate
+touch $RPM_BUILD_ROOT/var/lib/logrotate.status
 
 %clean
 rm -rf $RPM_BUILD_ROOT
@@ -41,10 +51,75 @@
 %attr(0755, root, root) /usr/sbin/logrotate
 %attr(0644, root, root) %{_mandir}/man8/logrotate.8*
 %attr(0755, root, root) /etc/cron.daily/logrotate
-%attr(0644, root, root) %config /etc/logrotate.conf
+%attr(0644, root, root) %config(noreplace) /etc/logrotate.conf
 %attr(0755, root, root) %dir /etc/logrotate.d
+%attr(0644, root, root) %verify(not size md5 mtime) %config(noreplace) /var/lib/logrotate.status
 
 %changelog
+* Mon Jan 26 2004 Dan Walsh <dwalsh@redhat.com> 3.6.10-4
+- fix is_selinux_enabled call
+
+* Fri Sep 5 2003 Dan Walsh <dwalsh@redhat.com> 3.6.10-3
+- Turn off selinux
+
+* Fri Sep 5 2003 Dan Walsh <dwalsh@redhat.com> 3.6.10-2.sel
+- Turn on selinux
+
+* Wed Aug 06 2003 Erik Troan <ewt@redhat.com>
+- always use compressext for the extension for compressed
+  files; before compresscmd and compressext had to agree
+- moved all compression to one code block
+- compression, scripts don't use system() anymore
+- compress and maillast didn't work together properly
+- delaycompress and mailfirst didn't work properly
+- don't use system() for mailing (or uncompressing) logs anymore
+- use "-s" for speciying the subjected of mailed logs
+
+* Thu Jul 24 2003 Elliot Lee <sopwith@redhat.com> 3.6.10-1
+- Fix #100546, change selinux port.
+
+* Wed Jul 18 2003 Dan Walsh <dwalsh@redhat.com> 3.6.9-2
+- Port to SELinux 2.5
+
+* Wed Jul 09 2003 Elliot Lee <sopwith@redhat.com> 3.6.9-1
+- Fix #90229, #90274, #89458, #91408
+
+* Mon Jan 20 2003 Elliot Lee <sopwith@redhat.com> 3.6.8-1
+- Old patch from pm@debian.org
+
+* Tue Jan 14 2003 Elliot Lee <sopwith@redhat.com> 3.6.7-1
+- Fixes from bugzilla
+
+* Fri Nov 15 2002 Elliot Lee <sopwith@redhat.com> 3.6.6-1
+- Commit patch from Fidelis Assis <fidelis@embratel.net.br>
+
+* Thu Jun 20 2002 Elliot Lee <sopwith@redhat.com> 3.6.5-1
+- Commit fix for #65299
+
+* Mon Apr 15 2002 Elliot Lee <sopwith@redhat.com> 3.6.4-1
+- Commit fix for #62560
+
+* Wed Mar 13 2002 Elliot Lee <sopwith@redhat.com> 3.6.3-1
+- Apply various bugfix patches from the openwall people
+
+* Tue Jan 29 2002 Elliot Lee <sopwith@redhat.com> 3.6.2-1
+- Fix bug #55809 (include logrotate.status in %files)
+- Fix bug #58328 (incorrect error detection when reading state file)
+- Allow 'G' size specifier from bug #57242
+
+* Mon Dec 10 2001 Preston Brown <pbrown@redhat.com>
+- noreplace config file
+
+* Wed Nov 28 2001 Preston Brown <pbrown@redhat.com> 3.6-1
+- patch from Alexander Kourakos <awk@awks.org> to stop the shared
+  postrotate/prerotate scripts from running if none of the log(s) need
+  rotating.  All log files are now checked for rotation in one batch,
+  rather than sequentially.
+- more fixes from Paul Martin <pm@debian.org>
+
+* Thu Nov  8 2001 Preston Brown <pbrown@redhat.com> 3.5.10-1
+- fix from paul martin <pm@debian.org> for zero-length state files
+
 * Tue Sep  4 2001 Preston Brown <pbrown@redhat.com>
 - fix segfault when logfile is in current directory.
 

=== modified file 'test/mailer'
--- test/mailer	2002-04-23 23:16:43 +0000
+++ test/mailer	2004-06-11 13:51:34 +0000
@@ -1,6 +1,4 @@
 #!/bin/bash
 
-echo "To: $1" >> mail-out
-echo "---" >> mail-out
+echo "$*" > mail-out
 cat >> mail-out
-echo "---" >> mail-out

=== modified file 'test/test'
--- test/test	2002-04-23 23:16:43 +0000
+++ test/test	2004-06-11 13:51:34 +0000
@@ -6,9 +6,11 @@
 RLR="$LOGROTATE $M $S"
 
 cleanup() {
-    rm -f test*.log* state test-config.[1-9] scriptout mail-out
+    rm -f test*.log* anothertest*.log* state test-config. scriptout mail-out
+    rm -f $(ls | egrep '^test-config.[0-9]+$')
 
     [ -n "$1" ] && echo "Running test $1"
+    return 0
 }
 
 genconfig() {
@@ -20,6 +22,7 @@
 createlog() {
     num=$1
     file=$2
+    cl_compressed=$3
 
     case $num in
 	0)
@@ -40,17 +43,31 @@
 	5)
 	    what=fifth
 	    ;;
+	6)
+	    what=sixth
+	    ;;
+	7)
+	    what=seventh
+	    ;;
+	8)
+	    what=eight
+	    ;;
+	9)
+	    what=ninth
+	    ;;
 	*)
 	    exit 1
 	    ;;
     esac
 
     echo $what > $file
+    [ -n "$cl_compressed" ] && gzip -9 $file
 }
 
 createlogs() {
     base=$1
     numlogs=$2
+    cls_compressed=$3
 
     rm -f ${base}*
 
@@ -59,37 +76,46 @@
 	if [ $num = 0 ]; then
 	    createlog 0 $base
 	else
-	    createlog $num ${base}.$num
+	    createlog $num ${base}.$num $cls_compressed
 	fi
 
 	num=`expr $num + 1`
     done
 }
 
+checkmail() {
+    (echo -s $PWD/$1 user@myhost.org; echo $2) | diff -u - mail-out
+}
+
 checkoutput() {
     while read line; do
 	set $line
 	file=$1
-	shift
+	co_compressed=$2
+	shift 2
 
 	fileother=`echo $line | awk '{print $1}'`
-	expected=`echo $line | cut -s -d\  -f2-`
+	expected=`echo $line | cut -s -d\  -f3-`
 
 	if [ $file != $fileother ]; then
 	    echo "unexpected file $file'" >&2
-	    return 1
+	    exit 2
 	fi
 
 	if [ ! -f $file ]; then
 	    echo "file $file does not exist"
 	fi
 
-	contents=`cat $file`
+	if [ -n "$co_compressed" -a "$co_compressed" != 0 ]; then
+		contents=`gunzip -c $file`
+	else
+		contents=`cat $file`
+	fi
 	if [ "$contents" != "$expected" ]; then
-	    echo "file $file does not contain expected results" >&2
+	    echo "file $file does not contain expected results (compressed $co_compressed, args $*)" >&2
 	    echo contains: \'$contents\'
 	    echo expected: \'$expected\'
-	    return 1
+	    exit 2
 	fi
     done
 }
@@ -98,12 +124,13 @@
     base=$1
     confignum=$2
     numlogs=$3
+    pt_compressed=$4
 
     rm -f $base*
     rm -f state
 
     genconfig $confignum
-    createlogs $base $numlogs
+    createlogs $base $numlogs $pt_compressed
 }
 
 # we don't want any stuff left from previous runs
@@ -113,9 +140,9 @@
 preptest test.log 1 2
 $RLR test-config.1 
 
-checkoutput test.log* <<EOF
-test.log zero
-test.log.1 first
+checkoutput <<EOF
+test.log 0 zero
+test.log.1 0 first
 EOF
 
 # Put in place a state file that will force a rotation
@@ -126,19 +153,19 @@
 
 # Now force the rotation
 $RLR test-config.1
-checkoutput test.log* <<EOF
-test.log
-test.log.1 zero
-test.log.2 first
+checkoutput <<EOF
+test.log 0
+test.log.1 0 zero
+test.log.2 0 first
 EOF
 
 # rerun it to make sure nothing happens
 $RLR test-config.1 
 
-checkoutput test.log* <<EOF
+checkoutput <<EOF
 test.log
-test.log.1 zero
-test.log.2 first
+test.log.1 0 zero
+test.log.2 0 first
 EOF
 
 cleanup 1
@@ -146,11 +173,13 @@
 preptest test.log 2 3
 $RLR test-config.2 --force
 
-checkoutput test.log* <<EOF
-test.log.1 zero
-test.log.2 first
+checkoutput <<EOF
+test.log.1 0 zero
+test.log.2 0 first
 EOF
 
+checkmail test.log.3 second
+
 if [ -f test.log ]; then
     echo "erroneously created test.log"
 fi
@@ -160,10 +189,10 @@
 preptest test.log 3 1
 $RLR test-config.3 --force
 
-checkoutput test.log* <<EOF
-test.log
-test.log.1 zero
-scriptout foo
+checkoutput <<EOF
+test.log 0
+test.log.1 0 zero
+scriptout 0 foo
 EOF
 
 cleanup 3
@@ -172,12 +201,12 @@
 preptest test2.log 3 1
 $RLR test-config.3 --force
 
-checkoutput test*.log* <<EOF
-test.log
-test.log.1 zero
-test2.log
-test2.log.1 zero
-scriptout foo foo
+checkoutput <<EOF
+test.log 0
+test.log.1 0 zero
+test2.log 0
+test2.log.1 0 zero
+scriptout 0 foo foo
 EOF
 
 cleanup 4
@@ -186,12 +215,12 @@
 preptest test2.log 4 1
 $RLR test-config.4 --force 
 
-checkoutput test*.log* <<EOF
-test.log
-test.log.1 zero
-test2.log
-test2.log.1 zero
-scriptout foo
+checkoutput <<EOF
+test.log 0
+test.log.1 0 zero
+test2.log 0
+test2.log.1 0 zero
+scriptout 0 foo
 EOF
 
 cleanup 5
@@ -200,13 +229,129 @@
 preptest anothertest.log 5 1
 $RLR test-config.5 --force 
 
-checkoutput test*.log* <<EOF
-test.log
-test.log.1 zero
-anothertest.log
-anothertest.log.1 zero
-scriptout foo
+checkoutput <<EOF
+test.log 0
+test.log.1 0 zero
+anothertest.log 0
+anothertest.log.1 0 zero
+scriptout 0 foo
 EOF
 
 cleanup 6
 
+preptest test.log 6 1
+preptest anothertest.log 6 1
+$RLR test-config.6 --force
+
+checkoutput <<EOF
+test.log 0
+test.log.0 0 zero
+anothertest.log 0
+anothertest.log.0 0 zero
+scriptout 0 foo
+EOF
+
+cleanup 7
+
+preptest test.log 7 1
+preptest anothertest.log 7 1
+
+$RLR test-config.7 --force
+
+checkoutput <<EOF
+test.log 0
+test.log.6 0 zero
+anothertest.log 0
+anothertest.log.6 0 zero
+scriptout 0 foo
+EOF
+
+cleanup 8
+
+preptest test.log 8 1 1
+$RLR test-config.8 --force
+
+checkoutput <<EOF
+test.log 0
+test.log.1.gz 1 zero
+scriptout 0 foo
+EOF
+
+checkmail test.log zero
+
+cleanup 9
+
+preptest test.log 9 1 1
+$RLR test-config.9 --force
+
+checkoutput <<EOF
+test.log 0
+scriptout 0 foo
+EOF
+
+checkmail test.log zero
+
+cleanup 10
+
+preptest test.log 10 1
+$RLR test-config.10 --force
+
+checkoutput <<EOF
+test.log 0
+test.log.1 0 zero
+EOF
+
+echo "newfile" > test.log
+
+$RLR test-config.10 --force
+checkoutput <<EOF
+test.log 0
+test.log.1 0 newfile
+test.log.2.gz 1 zero
+EOF
+
+checkmail test.log.1 newfile
+
+cleanup 11
+
+preptest test.log 11 2 1
+$RLR test-config.11 --force
+
+checkoutput <<EOF
+test.log 0
+scriptout 0 foo
+EOF
+
+checkmail test.log first
+
+# check rotation into a directory given as a relative pathname
+cleanup 12
+
+preptest test.log 12 1 0
+rm -rf testdir
+mkdir testdir
+$RLR test-config.12 --force
+
+checkoutput <<EOF
+test.log 0
+testdir/test.log.1 0 zero
+EOF
+
+rm -rf testdir
+
+# check rotation into a directory given as an absolute  pathname
+cleanup 13
+
+preptest test.log 13 1 0
+rm -rf testdir
+mkdir testdir
+$RLR test-config.13 --force
+
+checkoutput <<EOF
+test.log 0
+testdir/test.log.1 0 zero
+EOF
+
+rm -rf testdir
+
+cleanup

=== modified file 'test/test-config.1.in'
--- test/test-config.1.in	2002-04-23 23:16:43 +0000
+++ test/test-config.1.in	2004-06-11 13:51:34 +0000
@@ -1,4 +1,3 @@
-errors root@myhost.org
 create
 
 &DIR&/test.log {

=== added file 'test/test-config.10.in'
--- test/test-config.10.in	1970-01-01 00:00:00 +0000
+++ test/test-config.10.in	2004-06-11 13:51:34 +0000
@@ -0,0 +1,11 @@
+create
+
+&DIR&/test.log {
+    daily
+    rotate 3
+    compress
+    delaycompress
+    create
+    mailfirst
+    mail user@myhost.org
+}

=== added file 'test/test-config.11.in'
--- test/test-config.11.in	1970-01-01 00:00:00 +0000
+++ test/test-config.11.in	2004-06-11 13:51:34 +0000
@@ -0,0 +1,17 @@
+create
+
+compress
+
+&DIR&/test.log {
+    monthly
+    rotate 1
+    mail user@myhost.org
+    maillast
+    sharedscripts
+
+    postrotate
+	touch scriptout
+	echo $(cat scriptout) foo > foo
+	mv foo scriptout
+    endscript
+}

=== added file 'test/test-config.12.in'
--- test/test-config.12.in	1970-01-01 00:00:00 +0000
+++ test/test-config.12.in	2004-06-11 13:51:34 +0000
@@ -0,0 +1,7 @@
+create
+
+&DIR&/test.log {
+    monthly
+    rotate 1
+    olddir testdir
+}

=== added file 'test/test-config.13.in'
--- test/test-config.13.in	1970-01-01 00:00:00 +0000
+++ test/test-config.13.in	2004-06-11 13:51:34 +0000
@@ -0,0 +1,7 @@
+create
+
+&DIR&/test.log {
+    monthly
+    rotate 1
+    olddir &DIR&/testdir
+}

=== modified file 'test/test-config.2.in'
--- test/test-config.2.in	2002-04-23 23:16:43 +0000
+++ test/test-config.2.in	2004-06-11 13:51:34 +0000
@@ -1,5 +1,3 @@
-errors root@myhost.org
-
 "&DIR&/test.log" {
     monthly
     rotate 2

=== modified file 'test/test-config.3.in'
--- test/test-config.3.in	2002-04-23 23:16:43 +0000
+++ test/test-config.3.in	2004-06-11 13:51:34 +0000
@@ -1,4 +1,3 @@
-errors root@myhost.org
 create
 
 &DIR&/test*.log {

=== modified file 'test/test-config.4.in'
--- test/test-config.4.in	2002-04-23 23:16:43 +0000
+++ test/test-config.4.in	2004-06-11 13:51:34 +0000
@@ -1,4 +1,3 @@
-errors root@myhost.org
 create
 
 &DIR&/test*.log {

=== modified file 'test/test-config.5.in'
--- test/test-config.5.in	2002-04-23 23:16:43 +0000
+++ test/test-config.5.in	2004-06-11 13:51:34 +0000
@@ -1,4 +1,3 @@
-errors root@myhost.org
 create
 
 &DIR&/test.log &DIR&/anothertest.log {

=== added file 'test/test-config.6.in'
--- test/test-config.6.in	1970-01-01 00:00:00 +0000
+++ test/test-config.6.in	2004-06-11 13:51:34 +0000
@@ -0,0 +1,16 @@
+create
+
+&DIR&/test.log &DIR&/anothertest.log {
+    monthly
+    rotate 1
+    start 0
+    mail user@myhost.org
+    maillast
+    sharedscripts
+
+    postrotate
+	touch scriptout
+	echo $(cat scriptout) foo > foo
+	mv foo scriptout
+    endscript
+}

=== added file 'test/test-config.7.in'
--- test/test-config.7.in	1970-01-01 00:00:00 +0000
+++ test/test-config.7.in	2004-06-11 13:51:34 +0000
@@ -0,0 +1,16 @@
+create
+
+&DIR&/test.log &DIR&/anothertest.log {
+    monthly
+    rotate 3
+    start 6
+    mail user@myhost.org
+    maillast
+    sharedscripts
+
+    postrotate
+	touch scriptout
+	echo $(cat scriptout) foo > foo
+	mv foo scriptout
+    endscript
+}

=== added file 'test/test-config.8.in'
--- test/test-config.8.in	1970-01-01 00:00:00 +0000
+++ test/test-config.8.in	2004-06-11 13:51:34 +0000
@@ -0,0 +1,17 @@
+create
+
+compress
+
+&DIR&/test.log {
+    monthly
+    rotate 3
+    mail user@myhost.org
+    mailfirst
+    sharedscripts
+
+    postrotate
+	touch scriptout
+	echo $(cat scriptout) foo > foo
+	mv foo scriptout
+    endscript
+}

=== added file 'test/test-config.9.in'
--- test/test-config.9.in	1970-01-01 00:00:00 +0000
+++ test/test-config.9.in	2004-06-11 13:51:34 +0000
@@ -0,0 +1,17 @@
+create
+
+compress
+
+&DIR&/test.log {
+    monthly
+    rotate 0
+    mail user@myhost.org
+    mailfirst
+    sharedscripts
+
+    postrotate
+	touch scriptout
+	echo $(cat scriptout) foo > foo
+	mv foo scriptout
+    endscript
+}

